! 2012: This set of code (in the order of basemodel.f90, lib1.f90, lib2.f90) forms VS2008 project "fortrancode"
! Dependencies: IMSL 6.0 Fortran libraries (e.g. uminf_int, u4inf_int, anordf_int, chidf_int, linrg_int, fdjac_int, mxtxf_int);
! Intel MKL 10.1 (e.g. mkl_vsl, mkl_vsl_type, mkl95_blas)


INCLUDE 'link_fnl_shared.h'   ! Use IMSL 6.0 Fortran libraries interface
!include 'link_f90_dll_smp.h'  ! Use IMSL 5.0 Fortran libraries interface
program test
    use likeparam           ! load likelihood estimation global variables
    use platform            ! load platform-specific global variables
    USE SUBROUTINES         ! load main set of subroutines
    USE DATASTORE           ! load global variables
    USE uminf_INT           ! load multivariate optimization routine
    USE U4INF_INT           ! load optimization initialization routine
    use mkl_vsl_type        ! will define new streams in simulation
    use mkl_vsl             ! will define new streams in simulation
    implicit none   
    REAL(KIND=8)    ::  rparam(7), fscale=1.d0          ! optimization routine parameters
    real(kind=8), DIMENSION(paranummax)    ::  xguess, solution, parastd, xscale=1.d0    ! optimization routine parameters    
    real(kind=8), DIMENSION(paranummax,paranummax) ::  covmat
    real(kind=8)    ::  solutionmat(paranummax,4)
    character(3)   ::  solutionmatstar(paranummax)
    INTEGER    ::  long=0, iostatarr(20)     
    NAMELIST / instruction / parain    
    open(1, file=trim(path)//'path.txt', status='old',iostat=iostat)     ! Read path.txt
    read(1,*) path
    write(*,*) 'Input root path (enter slash for default path (VS fortran project directory):'     ! Read root path from standard input or external file
    write(*,*) 'Currently the path is: ', path
    read(*,*) path
    write(*,*) 'You have entered: ', path
    open(1, file=trim(path)//'instruction.txt', status='old',iostat=iostat)     ! Read instruction.txt
    print *, 'iostat for opening instruction.txt = ', iostat    
    READ(1,NML=instruction)
    close(1)    
    CALL checkparamvalue(parain)
    write (*,*) 'Values of simstatus, cdynprog, cjob, ctype, cdisc, cquad:'
    write (*,'(100(I1,1X))') parain%simstatus, parain%cdynprog, parain%cjob, parain%cdisc, parain%cquad
    
    CALL CPU_TIME(tstart)          ! record start time
    CALL U4INF(iparam, rparam)     ! call optimization initialization routine and reset optimization parameters
    rparam(1)=100.e0*rparam(1)      ! scaled gradient tolerance
    rparam(2)=100.e0*rparam(2)      ! scaled step tolerance
    iparam(3)=100000                ! maximum number of iterations (default 100)
    iparam(4)=10000000              ! maximum number of function evaluations (default 400)
    iparam(5)=1000000               ! maximum number of gradient evaluations (default 400)
    
    ! read model parameters from file (variable "readin" defined in datastore)
    ! parameter order: a) choice error(1) b) choice mean(7) c) wage mean(1) d) wage stderr(1) e) job offer mean(1)
    !   e) work, afdc, fs, wage, job, sanction equations f) type probability g) factors: work, afdc, fs, wage, job 
    paranum=parain%paranum
    nthread=parain%nthread
    open(1, file=trim(path)//'parainit.txt', status='old',iostat=iostat)
    print *, 'iostat for opening parainit.txt = ', iostat    
    read(1,*) (xguess(iloop),iloop=1,paranum)             
    close(1)           
    
    ! read datasets
    open(0, file=trim(path)//'gdata0.txt', status='old',iostat=iostatarr(1))           ! group-level data
    open(1, file=trim(path)//'gdata1.txt', status='old',iostat=iostatarr(2))           ! choice data
    open(2, file=trim(path)//'gdata2.txt', status='old',iostat=iostatarr(3))           ! wage data
    open(3, file=trim(path)//'gdata3.txt', status='old',iostat=iostatarr(4))           ! program parameter data
    open(4, file=trim(path)//'gdata4.txt', status='old',iostat=iostatarr(5))           ! other data used for miscell purposes (e.g. calendar year)
    open(5, file=trim(path)//'gdata5.txt', status='old',iostat=iostatarr(6))           ! minimum wage data
    open(6, file=trim(path)//'gdata6.txt', status='old',iostat=iostatarr(7))           ! number of kids under 18 data
    open(7, file=trim(path)//'gdata7.txt', status='old',iostat=iostatarr(8))           ! cumulative afdc and hit data
    open(41, file=trim(path)//'gdata4afdc.txt', status='old',iostat=iostatarr(9))      ! welfare preference variables
    open(42, file=trim(path)//'gdata4fs.txt', status='old',iostat=iostatarr(10))        ! food stamp preference variables
    open(43, file=trim(path)//'gdata4wage.txt', status='old',iostat=iostatarr(11))      ! wage equation variables
    open(44, file=trim(path)//'gdata4job.txt', status='old',iostat=iostatarr(12))       ! job offer variables
    open(45, file=trim(path)//'gdata4sanction.txt', status='old',iostat=iostatarr(13))  ! work requirement sanction variables
    open(46, file=trim(path)//'gdata4work.txt', status='old',iostat=iostatarr(14))      ! work preference variables
    open(51, file=trim(path)//'gdata0id.txt', status='old',iostat=iostatarr(15))        ! sample id data
    PRINT *,'IOSTAT ARRAY for reading data files ='
    WRITE (*,'(20(I2,1X))') iostatarr
    int1=0          ! Count number of observations read from data file
    do iloop=1,parain%groupsize        
        read(0,'(100i3)') gdatain(iloop)%gsize, gdatain(iloop)%horizon, gdatain(iloop)%tlcompute, &
            gdatain(iloop)%tlstart, gdatain(iloop)%tllen, gdatain(iloop)%tlhor, gdatain(iloop)%tlred, gdatain(iloop)%tlnlife, gdatain(iloop)%tlnlifebase
        read(51,'(i8)') gdatain(iloop)%id        
        do jloop=1,gdatain(iloop)%gsize
            int1=int1+1        
            read(1,'(i2)') gdatain(iloop)%datain(jloop)%choice
            read(2,'(100f13.7)') gdatain(iloop)%datain(jloop)%wage
            read(3,'(100f13.7)') gdatain(iloop)%datain(jloop)%afdc, gdatain(iloop)%datain(jloop)%fs, gdatain(iloop)%datain(jloop)%eitc, &
                gdatain(iloop)%datain(jloop)%ftax, gdatain(iloop)%datain(jloop)%sttax
            read(4,'(100f13.7)') gdatain(iloop)%datain(jloop)%var(1)    ! read calendar year
            read(5,'(100f13.7)') gdatain(iloop)%datain(jloop)%minwage
            read(6,'(100f13.7)') gdatain(iloop)%datain(jloop)%rfoklt18
            read(7,'(100i3)') gdatain(iloop)%datain(jloop)%cumafdc, gdatain(iloop)%datain(jloop)%hit, gdatain(iloop)%datain(jloop)%cumhitwait
            read(41,'(100f13.7)') gdatain(iloop)%datain(jloop)%varafdc
            read(42,'(100f13.7)') gdatain(iloop)%datain(jloop)%varfs
            read(43,'(100f13.7)') gdatain(iloop)%datain(jloop)%varwage
            read(44,'(100f13.7)') gdatain(iloop)%datain(jloop)%varjob
            read(45,'(100f13.7)') gdatain(iloop)%datain(jloop)%sanction
            read(46,'(100f13.7)') gdatain(iloop)%datain(jloop)%varwork
            gdatain(iloop)%datain(jloop)%varwage=gdatain(iloop)%datain(jloop)%varwage/10.d0     ! normalize wage equation variables
            gdatain(iloop)%datain(jloop)%varjob=gdatain(iloop)%datain(jloop)%varjob/10.d0       ! normalize job offer variables
            gdatain(iloop)%datain(jloop)%tlred=gdatain(iloop)%tlred                                       
                                   
            ! Counterfactual: change policy parameters (cftype=0,2,3,...,10)
            IF (parain%cftype==2) then
                gdatain(iloop)%datain(jloop)%afdc(1)=gdatain(iloop)%datain(jloop)%afdc(1)+100.d0    ! increase welfare elig threshold
                gdatain(iloop)%datain(jloop)%afdc(2)=gdatain(iloop)%datain(jloop)%afdc(2)+100.d0    ! increase welfare guarantee
                gdatain(iloop)%datain(jloop)%afdc(3)=gdatain(iloop)%datain(jloop)%afdc(3)+100.d0    ! increase welfare max benefit
            ELSEIF (parain%cftype==3) then
                gdatain(iloop)%datain(jloop)%afdc(1)=gdatain(iloop)%datain(jloop)%afdc(1)+100.d0    ! increase welfare elig threshold
                gdatain(iloop)%datain(jloop)%afdc(5)=0.d0           ! percent disregards set to 100 percent
            ELSEIF (parain%cftype==4) then
                gdatain(iloop)%datain(jloop)%fs(1)=gdatain(iloop)%datain(jloop)%fs(1)+50.d0        ! food stamps gross income elig threshold
                gdatain(iloop)%datain(jloop)%fs(2)=gdatain(iloop)%datain(jloop)%fs(2)+50.d0        ! net income elig threshold
                gdatain(iloop)%datain(jloop)%fs(3)=gdatain(iloop)%datain(jloop)%fs(3)+50.d0        ! max benefit level
            ELSEIF (parain%cftype==5) then
                parain%fsrate=0.1d0            ! decrease food stamp benefit reduction rate
            ELSEIF (parain%cftype==6) then
                gdatain(iloop)%datain(jloop)%eitc(4)=gdatain(iloop)%datain(jloop)%eitc(4)*1.2d0     ! increase maximum benefit
                gdatain(iloop)%datain(jloop)%eitc(5)=gdatain(iloop)%datain(jloop)%eitc(5)*1.2d0     ! increase phase-in rate
                gdatain(iloop)%datain(jloop)%eitc(6)=gdatain(iloop)%datain(jloop)%eitc(6)*0.8d0     ! decrease phase-out rate
            ELSEIF (parain%cftype==7) then            
                gdatain(iloop)%datain(jloop)%eitc(4)=gdatain(iloop)%datain(jloop)%eitc(4)*1.4d0     ! increase maximm benefit
                gdatain(iloop)%datain(jloop)%eitc(5)=gdatain(iloop)%datain(jloop)%eitc(5)*1.4d0     ! increase phase-in rate
                gdatain(iloop)%datain(jloop)%eitc(2)=gdatain(iloop)%datain(jloop)%eitc(1)+1.d0     ! elig level 2
            ELSEIF (parain%cftype==8) then
                gdatain(iloop)%datain(jloop)%eitc(1)=gdatain(iloop)%datain(jloop)%eitc(1)*1.5d0     ! elig level 1
                gdatain(iloop)%datain(jloop)%eitc(4)=gdatain(iloop)%datain(jloop)%eitc(4)*1.2d0     ! increase maximum benefit
                gdatain(iloop)%datain(jloop)%eitc(5)=gdatain(iloop)%datain(jloop)%eitc(5)*0.8d0     ! decrease phase-in rate
                gdatain(iloop)%datain(jloop)%eitc(6)=gdatain(iloop)%datain(jloop)%eitc(6)*0.3d0     ! decrease phase-out rate
            !ELSEIF (parain%cftype==8) then
            !    gdatain(iloop)%datain(jloop)%sttax(22)=0.d0         ! no state EITC
            ELSEIF (parain%cftype==9) then
                parain%tprate=0.042d0            ! decrease payroll tax rate
            ELSEIF (parain%cftype==10) then                
                gdatain(iloop)%datain(jloop)%ftax(2)=gdatain(iloop)%datain(jloop)%ftax(2)*1.28d0     ! increase federal tax exemption
            ELSEIF (parain%cftype==11) then    
                gdatain(iloop)%datain(jloop)%sttax(1)=0.d0          ! no state income tax
            END IF
            
            ! Long-run simulation
            IF (parain%lcftype==2) then    ! 5-year lifetime limit
                gdatain(iloop)%tlcompute=1        
                gdatain(iloop)%tlnlife=0
                gdatain(iloop)%tllen=15
            ELSEIF (parain%lcftype==3) then    ! 5-year lifetime limit myopia
                gdatain(iloop)%tlcompute=0           
                gdatain(iloop)%tlnlife=0
                gdatain(iloop)%tllen=15      
            ELSEIF (parain%lcftype==4) then   ! 4-4 nonlifetime limit
                gdatain(iloop)%tlcompute=1              
                gdatain(iloop)%tlnlife=1
                gdatain(iloop)%tllen=12
                gdatain(iloop)%tlnlifebase=12
            ELSEIF (parain%lcftype==5) then    ! reduce welfare benefit by 30 percent among recipients
            ELSEIF (parain%lcftype==6) then   ! intermediate sanction
                gdatain(iloop)%datain(jloop)%sanction(2)=1.d0
            ELSEIF (parain%lcftype==7) then   ! 1999 federal EITC
                IF (gdatain(iloop)%datain(jloop)%rfoklt18==0) then
                    gdatain(iloop)%datain(jloop)%eitc=(/ 377.50, 472.50, 850.00, 28.88, 0.0765, 0.0765 /)
                ELSE IF (gdatain(iloop)%datain(jloop)%rfoklt18==1) then
                    gdatain(iloop)%datain(jloop)%eitc=(/ 566.67, 1038.33, 2244.00, 192.67, 0.34, 0.1598 /)
                ELSE
                    gdatain(iloop)%datain(jloop)%eitc=(/ 795.00, 1038.33, 2548.33, 318.00, 0.40, 0.2106 /)
                END IF 
            ELSEIF (parain%lcftype==8) then   ! work subsidy
            ELSEIF (parain%lcftype==9) THEN  ! reduce unemployment rate by 1 percentage point
                parain%unempelast=1            
            ELSEIF (parain%lcftype==10) then  ! 5-year lifetime limit, intermed work requirement and EITC
                gdatain(iloop)%tlcompute=1
                gdatain(iloop)%tlnlife=0
                gdatain(iloop)%tllen=15
                gdatain(iloop)%datain(jloop)%sanction(2)=1.d0
                IF (gdatain(iloop)%datain(jloop)%rfoklt18==0) then
                    gdatain(iloop)%datain(jloop)%eitc=(/ 377.50, 472.50, 850.00, 28.88, 0.0765, 0.0765 /)
                ELSE IF (gdatain(iloop)%datain(jloop)%rfoklt18==1) then
                    gdatain(iloop)%datain(jloop)%eitc=(/ 566.67, 1038.33, 2244.00, 192.67, 0.34, 0.1598 /)
                ELSE
                    gdatain(iloop)%datain(jloop)%eitc=(/ 795.00, 1038.33, 2548.33, 318.00, 0.40, 0.2106 /)
                END IF                 
            ELSEIF (parain%lcftype==11) then   ! 3-5 nonlife limit (reference only)
                gdatain(iloop)%tlcompute=1              
                gdatain(iloop)%tlnlife=1
                gdatain(iloop)%tllen=9
                gdatain(iloop)%tlnlifebase=15
            ELSEIF (parain%lcftype==12) then   ! 5-5 nonlife limit (reference only)
                gdatain(iloop)%tlcompute=1              
                gdatain(iloop)%tlnlife=1
                gdatain(iloop)%tllen=15
                gdatain(iloop)%tlnlifebase=15
            ELSEIF (parain%lcftype==13) then   ! 3-3 nonlife limit (reference only)
                gdatain(iloop)%tlcompute=1              
                gdatain(iloop)%tlnlife=1
                gdatain(iloop)%tllen=9
                gdatain(iloop)%tlnlifebase=9
            ELSEIF (parain%lcftype==14) then   ! work subsidy type 2
            ELSEIF (parain%lcftype==15) then    ! interm work requirement and EITC only
                gdatain(iloop)%datain(jloop)%sanction(2)=1.d0
                IF (gdatain(iloop)%datain(jloop)%rfoklt18==0) then
                    gdatain(iloop)%datain(jloop)%eitc=(/ 377.50, 472.50, 850.00, 28.88, 0.0765, 0.0765 /)
                ELSE IF (gdatain(iloop)%datain(jloop)%rfoklt18==1) then
                    gdatain(iloop)%datain(jloop)%eitc=(/ 566.67, 1038.33, 2244.00, 192.67, 0.34, 0.1598 /)
                ELSE
                    gdatain(iloop)%datain(jloop)%eitc=(/ 795.00, 1038.33, 2548.33, 318.00, 0.40, 0.2106 /)
                END IF                                 
            END IF          
            
            IF (parain%unempelast==1) THEN             
                IF (parain%cjob==1) then
                    gdatain(iloop)%datain(jloop)%varjob(parain%npjob-2:parain%npjob) = gdatain(iloop)%datain(jloop)%varjob(parain%npjob-2:parain%npjob)-0.1d0
                ELSE
                    gdatain(iloop)%datain(jloop)%varwork(parain%npwork-2:parain%npwork) = gdatain(iloop)%datain(jloop)%varwork(parain%npwork-2:parain%npwork)-0.1d0
                END IF
            END IF           
            
            IF (parain%nowr==1) gdatain(iloop)%datain(jloop)%sanction=0.d0       ! no work requirements
            IF (parain%minw92==1) gdatain(iloop)%datain(jloop)%minwage=340.d0       ! minimum wage set at 1992 level
        end do
        
        IF (parain%notl==1) gdatain(iloop)%tlcompute=0             ! no time limits
    end do
    
    IF (parain%lcftype .ne. 0) then        
    do iloop=1,parain%groupsize           
        gdatain(iloop)%gsize = gdatain(iloop)%horizon
        do jloop=1,gdatain(iloop)%gsize
            gdatain(iloop)%datain(jloop) = gdatain(iloop)%datain(1)
        end do
    end do
    end if
    
    print *, 'Number of observations read = ', int1
    close(0)
    close(1)
    close(2)
    close(3)    
    close(4)
    close(5)
    close(6)
    close(7)    
    close(41)
    close(42)
    close(43)
    close(44)
    close(45)
    close(46)
    close(51)    
    
    ! open file to record iterations of likelihood function (for likefcn only)
    open (1, file=trim(path)//'iterate.txt',status='replace')      
    write (1,*) 'Finished reading dataset and parameters. Number of model parameters =', paranum    
    write (*,*) 'Finished reading dataset and parameters. Number of model parameters =', paranum    
    write (1,*) 'Number of parallel threads =', nthread
    write (*,*) 'Number of parallel threads =', nthread

    if (parain%simstatus==3) then                      ! test program speed
        WRITE(*,*) 'Testing program speed. Number of likelihood evaluations =', parain%testspeed
        do iloop=1,parain%testspeed    
            print *,iloop        
            CALL likefcn(paranum, xguess(1:paranum), fscale)
        end do
    elseif (parain%simstatus==1) then                 ! launch optimization       
        CALL UMINF (LIKEFCN, X=SOLUTION(1:paranum), xguess=xguess(1:paranum), iparam=iparam, rparam=rparam, fscale=fscale, xscale=xscale(1:paranum))
        CALL std(solution(1:paranum),parastd(1:paranum), covmat(1:paranum,1:paranum), parain%groupsize, paranum)            
    elseif (parain%simstatus==0) then            ! given xguess, compute likelihood value and standard deviation of model parameters
        CALL likefcn(paranum, xguess(1:paranum), fscale)
        solution=xguess          
        CALL std(solution(1:paranum),parastd(1:paranum), covmat(1:paranum,1:paranum), parain%groupsize, paranum)       
    elseif (parain%simstatus==2) then            ! launch counterfactual simulation and write output
        open (14, file=trim(path)//'simulate4.txt',status='replace')
                
        IF (long==1) parain%discount=0.d0           ! ???
        CALL likefcn(paranum, xguess(1:paranum), fscale)       ! run likefcn once to remap parameters and launch dynprog (if instructed)
        do jloop=4,4        
            iostat = vslnewstream( parain%sstream, VSL_BRNG_MT2203, 801 )       ! GENERATE CHOICE
            if (iostat /= vsl_error_ok) print *, 'error new stream in SSTREAM'
            iostat = vslnewstream( parain%sstream2, VSL_BRNG_MT2203, 802 )      ! GENERATE FACTOR, WAGE OFFER, JOB OFFER
            if (iostat /= vsl_error_ok) print *, 'error new stream in SSTREAM2'            
            iostat = vslnewstream( parain%sstream3, VSL_BRNG_MT2203, 803 )      ! GENERATE FACTOR, WAGE OFFER, JOB OFFER
            if (iostat /= vsl_error_ok) print *, 'error new stream in SSTREAM3'
            DO iloop=1,parain%groupsize       
                CALL simulatetype(gdatain(iloop), parain, jloop, 10, 10+jloop,simin )    ! ind (default=4), simsize (default=10), writenum i/o file number (10 to 14)
            end do
        END DO          
        IF (parain%lcftype==0) CALL siminprocess(path,simin)        ! process and write summary statistics output
        goto 999
    end if   
    close(1)
    close(10)
    close(11)
    close(12)
    close(13)
    close(14)            

    ! Define solution matrix (parameter, standard error, t-value, pvalue)
    do iloop=1,paranum
        solutionmat(iloop,1)=solution(iloop)
        solutionmat(iloop,2)=parastd(iloop)
        solutionmat(iloop,3)=solution(iloop)/parastd(iloop)
        solutionmat(iloop,4)=pvalue(solution(iloop)/parastd(iloop),2)
        solutionmatstar(iloop)=pvaluestar(solutionmat(iloop,4))
    end do
    
    CALL CPU_TIME(tend)          ! record end time    
    timetotal=(tend-tstart)/DBLE(parain%nthread)
    
    ! Write likelihood value of each individual to file
    open(1,file=trim(path)//'likelihood.txt',status='replace')     
    do iloop=1,parain%groupsize
        write(1, '(i5, 1x, f13.7)') iloop, farr(iloop)
    end do
    close(1)
    
    ! Write solution to screen and files       
    open(1,file=trim(path)//'output.txt',status='replace')     
    write(*,*) 'Time of operation was ', INT(timetotal/60.d0), ' minutes', INT((timetotal/60.d0 - INT(timetotal/60.d0))*60.d0), ' seconds.'
    write(1,*) 'Time of operation was ', INT(timetotal/60.d0), ' minutes', INT((timetotal/60.d0 - INT(timetotal/60.d0))*60.d0), ' seconds.'
    write(*,*) 'Number of iterations and likelihood = ', NITER, currentmin
    write(1,*) 'Number of iterations and likelihood = ', NITER, currentmin
    write(*,*) 'Parameter #, estimate, standard error, t-value, p-value:'    
    write(1,*) 'Parameter #, estimate, standard error, t-value, p-value:'    
    do iloop=1,paranum
        write (*, '( I3, 4(f8.2, 1x), A3 )') iloop, (solutionmat(iloop,jloop), jloop=1,4), solutionmatstar(iloop)    
        write (1, '( I3, 4(f8.2, 1x), A3 )') iloop, (solutionmat(iloop,jloop), jloop=1,4), solutionmatstar(iloop)
    end do
    
    CALL deltamethod(parain%deltaindex,solution,covmat,parain%ctype)         ! print deltamethod output    
    close(1)        
    
    open(1,file=trim(path)//'output_covmat.txt',status='replace')     
    do iloop=1,paranum
        write (1, '( 100(f13.7, 1x) )') (covmat(iloop,jloop),jloop=1,paranum)
    end do
    close(1)
        
    open(1,file=trim(path)//'output_list.txt',status='replace')     
    write (1,*) 'high-precision estimate in list form:'    
    write (1, '(100(10(f13.7, 1x), /))') (solutionmat(iloop,1),iloop=1,paranum)
    write (1,*) 'high-precision standard error in list form:'
    write (1, '(100(10(f13.7, 1x), /))') (solutionmat(iloop,2),iloop=1,paranum)
    write (1,*) 'high-precision t value in list form:'
    write (1, '(100(10(f13.7, 1x), /))') (solutionmat(iloop,3),iloop=1,paranum)
    write (1,*) 'high-precision p value in list form:'
    write (1, '(100(10(f13.7, 1x), /))') (solutionmat(iloop,4),iloop=1,paranum)
    close(1)

    open(1,file=trim(path)//'output_parain.txt',status='replace')     
    write (1,*) 'Root path: ', path
    write (1,*) 'Parain object = '
    write (1,NML=instruction)                                      ! print full list of parameters in NAMELIST format
    close(1)    
            
    999 CALL BEEPQQ (4000,300)    
    CONTAINS
        SUBROUTINE deltamethod(index,solution,covmat,ctype)
            IMPLICIT NONE
            INTEGER,INTENT(IN)    ::  index,ctype
            REAL(KIND=8),INTENT(IN)    ::  solution(:), covmat(:,:)
            REAL(KIND=8)   ::  mat(2,2),cov(2,2),g1,g2,s1,s2            
            write(1,*) 'Delta method:'
            write(1,*) 'Choice error:'
            write(1,'(3(f13.7,1x))') EXP(solution(1)), parastd(1)*EXP(solution(1))
            write(1,*) 'log wage equation stddev:'
            write(1,'(3(f13.7,1x))') EXP(solution(10)), parastd(10)*EXP(solution(10))            
            IF (ctype==1) THEN
                g1=exp(solution(index))/(1.d0+exp(solution(index))+exp(solution(index+1)))
                g2=exp(solution(index+1))/(1.d0+exp(solution(index))+exp(solution(index+1)))            
                mat(1,1)=g1*(1.d0-g1)
                mat(1,2)=-g1*g2
                mat(2,1)=-g1*g2
                mat(2,2)=g2*(1.d0-g2)            
                cov=transpose(mat)*covmat(index:index+1,index:index+1)*mat
                s1=sqrt(cov(1,1))
                s2=sqrt(cov(2,2))            
                write(1,*) 'type probabilities:'
                write(1,'(3(f13.7,1x))') g1, s1
                write(1,'(3(f13.7,1x))') g2, s2
            END IF
        END SUBROUTINE deltamethod
            
end program test
