! Stores parameters and generic variables
MODULE DATASTORE
    use mkl_vsl_type, only: vsl_stream_state       ! load random stream object type
    use platform, only: iloop                      ! load this variable to define arrays only
    IMPLICIT NONE    
    CHARACTER(50)  ::  path=' '             ! root path for reading data and parameter files (default $(ProjectDir))    
    INTEGER,PARAMETER :: vardim=6, varworkdim=9+1, varafdcdim=14+1, varfsdim=11+1, varwagedim=10+1, varjobdim=11+1, sanctiondim=3, typenum=3, vartldim=3, pdiscount=1
    INTEGER,PARAMETER :: paranummax=11+varworkdim+varafdcdim+varfsdim+varwagedim+varjobdim+sanctiondim + 6*(typenum-1)+5    
    INTEGER            ::  paranum=0
    INTEGER,PARAMETER :: mdim=8, sample=100, groupsize=2253, maxgsize=54+1
    INTEGER,PARAMETER  ::  maxhorizon=18*3+1, maxtllen=15, maxnlife=15, statenum=30, ghqpoint=5, ghqpointdyn=3
    INTEGER    ::  TID, CHUNKSIZE=145, NTHREAD=4           ! thread id, sample size for each thread
    INTEGER, EXTERNAL ::  OMP_GET_THREAD_NUM              ! function, obtain thread number in computer (platform specific)
    REAL(KIND=8)    ::  farr(groupsize), readin(110)        ! likelihood array, number of model parameters to read from file
    TYPE (VSL_STREAM_STATE), dimension(8) :: stream       ! define an array of random stream object    
    TYPE   ::  dataobj
        INTEGER    ::  choice                      ! choice (1 to mdim)
        REAL(KIND=8)    ::  wage                    ! wage (non-workers has zero value)
        REAL(KIND=8)    ::  afdc(6)                 ! welfare parameter
        REAL(KIND=8)    ::  fs(4)                   ! food stamp parameter
        REAL(KIND=8)    ::  eitc(6)                 ! eitc parameter
        REAL(KIND=8)    ::  ftax(2)                 ! federal tax parameter
        REAL(KIND=8)    ::  var(vardim)             ! generic explanatory variables (not used)
        REAL(KIND=8)    ::  varwork(varworkdim)     ! work preference equation
        REAL(KIND=8)    ::  varafdc(varafdcdim)     ! welfare preference equation
        REAL(KIND=8)    ::  varfs(varfsdim)         ! food stamp preference equation
        REAL(KIND=8)    ::  varwage(varwagedim)     ! wage equation
        REAL(KIND=8)    ::  varjob(varjobdim)       ! job offer equation (logit)
        REAL(KIND=8)    ::  minwage     ! minimum wage
        REAL(KIND=8)    ::  rfoklt18    ! number of child under 18 (for geninc use only)
        REAL(KIND=8)    ::  sanction(sanctiondim)
        REAL(KIND=8)    ::  sttax(24)               ! state income tax and EITC parameter
        INTEGER    ::  tlred           ! benefit-reduction limit indicator (copied from gdataobj, for geninc only, default=0)
        INTEGER    ::  cumafdc         ! number of cumulative periods on welfare since time limit begins (default=1)
        INTEGER    ::  hit             ! hit time limit indicator (default=0)
        INTEGER    ::  cumhitwait      ! number of waited periods since hitting non-lifetime limit (default=1)
        INTEGER    ::  factor          ! store simulated factors (unobserved effects)        
    END TYPE dataobj
    TYPE   ::  gdataobj
        TYPE (dataobj) ::  datain(maxgsize)    ! source data (for likelihood computation)
        TYPE (dataobj) ::  evdatain            ! source data (for dynamic programming only)  
        REAL(KIND=8)    ::  evbase(typenum,mdim,ghqpointdyn,2,2,2,2)=0.d0   ! u_k (type,mdim,quadrature wage,hit indicator,work,afdc,fs) for emax computation
        REAL(KIND=8)    ::  evbaseno(typenum,mdim,2,2,2,2)=0.d0             ! u_k (no job offer)
        REAL(KIND=8)    ::  evjobprob(typenum,2,2,2)=0.d0                   ! job offer probability (type,work,afdc,fs) for emax computation
        REAL(KIND=8)    ::  ev(typenum,maxhorizon,maxtllen,2,2,2)=0.d0      ! ev (time limit) (type,time,M,work,afdc,fs)
        REAL(KIND=8)    ::  evhit(typenum,maxhorizon,maxnlife,2,2,2)=0.d0   ! ev (non-lifetime) (type,time,m,work,afdc,fs)
        REAL(KIND=8)    ::  evntl(typenum,maxhorizon,2,2,2)=0.d0            ! ev (no time limit) (type,time,work,afdc,fs)
        INTEGER    ::  gsize                   ! length of panel data
        INTEGER    ::  horizon                 ! length of time horizon (periods until youngest child becomes 18, reference period wave 1). For evntl only.
        INTEGER    ::  tlhor                   ! length of time horizon (periods until youngest child becomes 18, reference period tlstart). For ev and evhit.
        INTEGER    ::  tlcompute               ! Time limit existence indicator
        INTEGER    ::  tlstart                 ! Start wave of time limit (default=1)        
        INTEGER    ::  tllen                   ! Length of time limit
        INTEGER    ::  tlnlife                 ! Non-lifetime limit indicator
        INTEGER    ::  tlnlifebase             ! Length of benefit-waiting period (non-lifetime limit)
        INTEGER    ::  tlred                   ! Benefit reduction time limit indicator        
        INTEGER    ::  id                      ! Family ID (for i/o only)
    END TYPE gdataobj            
    TYPE (gdataobj) :: gdatain(groupsize)
    
    TYPE   ::  ghqobj
        REAL(KIND=8)    ::  euler=0.577215664901532860606512090082d0    
        INTEGER        ::  npointdyn=3
        REAL(KIND=8)    ::  abscidyn(3)=(/-1.22474487139d0, 0.d0, 1.22474487139d0 /)*SQRT(2.d0)
        REAL(KIND=8)    ::  weightdyn(3)=(/ 0.295408975151d0, 1.1816359006d0, 0.295408975151d0 /)/SQRT(3.1415926535897932384626433832795D0)           
        INTEGER        ::  npoint=5               
        REAL(KIND=8)    ::  absci(5)=(/-2.02018287046d0,	-0.958572464614d0,	0.d0,	0.958572464614d0,	2.02018287046d0 /)*SQRT(2.d0)
        REAL(KIND=8)    ::  weight(5)= (/ 0.019953242059d0,	0.393619323152d0,	0.945308720483d0,	&
                                0.393619323152d0,	0.019953242059d0 /)/SQRT(3.1415926535897932384626433832795D0)   
    END TYPE ghqobj        
    
    TYPE   ::  paraobj     
        REAL(KIND=8)    ::  errmean(mdim)=0.d0              ! choice mean
        REAL(KIND=8)    ::  errvar=1.d0                     ! choice variance        
        REAL(KIND=8)    ::  wstat(2)=(/ 0.d0, 1.d0 /)       ! wage mean and variance        
        REAL(KIND=8)    ::  jobprobmean=999.d0              ! job offer mean (logit)
        REAL(KIND=8)    ::  pvarwork(varworkdim)=0.d0       ! work preference parameters
        REAL(KIND=8)    ::  pvarafdc(varafdcdim)=0.d0       ! welfare preference parameters
        REAL(KIND=8)    ::  pvarfs(varfsdim)=0.d0           ! food stamp preference parameters
        REAL(KIND=8)    ::  pvarwage(varwagedim)=0.d0       ! wage equation parameters
        REAL(KIND=8)    ::  pvarwstd(varwagedim)=0.d0       ! wage variance parameters (not used)
        REAL(KIND=8)    ::  pvarjob(varjobdim)=0.d0         ! job offer parameters (logit)
        REAL(KIND=8)    ::  psanction(sanctiondim)=0.d0     ! work requirement sanction parameters
        REAL(KIND=8)    ::  pvar(mdim,vardim)=0.d0          ! not used
        REAL(KIND=8)    ::  pvarsh(4,vardim)=0.d0           ! not used        
        REAL(KIND=8)    ::  pvartl(vartldim)=0.d0           ! not used        
        REAL(KIND=8)    ::  typeprob(typenum)=(/1.d0, (0.d0, iloop=2,typenum) /)          ! type probability
        REAL(KIND=8)    ::  factorw(typenum)=0.d0           ! factor load (wage)
        REAL(KIND=8)    ::  factorj(typenum)=0.d0           ! factor load (job offer, logit)
        REAL(KIND=8)    ::  factorwork(typenum)=0.d0        ! factor load (work preference)
        REAL(KIND=8)    ::  factorafdc(typenum)=0.d0        ! factor load (welfare preference)
        REAL(KIND=8)    ::  factorfs(typenum)=0.d0          ! factor load (food stamp preference)
        REAL(KIND=8)    ::  discount=0.953d0                ! discount factor (default: 0.953 (annualized value 0.86552))
        REAL(KIND=8)    ::  quadratic=0.d0                  ! utility quadratic term
        REAL(KIND=8)    ::  nthread=4                       ! number of threads for OPENMP parallel construct
        INTEGER        ::  simstatus=-1                    ! directive: 1 launch optimization, 0 obtain standard errors, 2 counterfactual simulation.
        INTEGER        ::  testspeed=1000                  ! directive: number of likelihood evaluations when testing speed of program
        INTEGER        ::  cdynprog=-1                     ! directive (loaded in master program): 0 not launch dyn programming, 1 otherwise        
        INTEGER        ::  dpsdep=-1                       ! directive: 0 DP no state dependence; 1 DP with state dependence        
        INTEGER        ::  cjob=-1                         ! directive (loaded in master program): 0 no job offer parameters in model, 1 otherwise
        INTEGER        ::  csanction=-1                    ! directive (loaded in master program): 0 no sanction in model, 1 otherwise
        INTEGER        ::  ctype=-1                        ! directive (loaded in master program): 0 no multiple unobserved types in model, 1 otherwise
        INTEGER        ::  cdisc=-1                        ! directive (loaded in master program): 0 no discount rate estimation, 1 otherwise
        INTEGER        ::  cquad=-1                        ! directive (loaded in master program): 0 no quadratic utility estimation, 1 otherwise
        INTEGER        ::  cbig=-1                         ! 0 small model with intercepts only, 1 otherwise
        INTEGER        ::  npwork=1, npafdc=1, npfs=1, npwage=1, npjob=1, npsanction=1, nptype=1   ! number of parameters estimated in model
        INTEGER        ::  paranum=-1
        INTEGER        ::  dirsim=0
        REAL(KIND=8)    ::  afdcfrac=0.d0                   ! not used
        REAL(KIND=8)    ::  incfrac(6)=1.d0                 ! not used
        INTEGER    ::  mdim=mdim
        INTEGER    ::  sample=sample
        INTEGER    ::  groupsize=groupsize
        INTEGER    ::  maxgsize=maxgsize
        INTEGER    ::  typenum=typenum        
        INTEGER    ::  vardim=vardim       ! not used
        INTEGER    ::  varworkdim=varworkdim
        INTEGER    ::  varafdcdim=varafdcdim
        INTEGER    ::  varfsdim=varfsdim
        INTEGER    ::  varwagedim=varwagedim
        INTEGER    ::  varjobdim=varjobdim     
        INTEGER    ::  sanctiondim=sanctiondim
        TYPE(ghqobj)  ::   ghq 	         
        TYPE(vsl_stream_state) ::  sstream                 ! random stream
        TYPE(vsl_stream_state) ::  sstream2                ! random stream
        TYPE(vsl_stream_state) ::  sstream3                ! random stream
        INTEGER   ::   deltaindex=0                          ! delta method index for type probabilities
        INTEGER    ::  welast=0                         ! directive: increase wage by 10 percent permanently
        INTEGER    ::  jelast=0                         ! directive: increase job offer prob by 10 percent permanently
        INTEGER    ::  welasttmp=0                      
        INTEGER    ::  jelasttmp=0                      
        INTEGER    ::  cftype=0                         ! directive: change benefit formula parameters 
        REAL(KIND=8)    ::  fsrate=0.3d0                    ! default food stamp formula benefit reduction rate
        REAL(KIND=8)    ::  tprate=0.0765d0                 ! default payroll tax rate
        INTEGER   ::   writesim=1                          ! directive: write simulation output to external file
        INTEGER    ::  notl=0                              ! directive: no time limit
        INTEGER    ::  nowr=0                              ! directive: no work requirements
        INTEGER    ::  minw92=0                         ! directive: minimum wage set at 1992 level
        INTEGER    ::  lcftype=0                        ! directive: long-run simulation. default lcftype=0 (no long-run simulation)
        INTEGER    ::  unempelast=0                     ! Increase unemployment rates by 1 percentage point
    END TYPE paraobj
    TYPE (paraobj) ::  parain, paraoutstd, paraouttstat, paraoutpval
    
    TYPE   ::  lcobj
        REAL(KIND=8)    ::  wage
        INTEGER    ::  choice
        REAL(KIND=8)    ::  lincom(mdim)
        REAL(KIND=8)    ::  wlincom
        REAL(KIND=8)    ::  wstdlincom
        REAL(KIND=8)    ::  jlincom
        REAL(KIND=8)    ::  lincomf(mdim)
        REAL(KIND=8)    ::  wlincomf
        REAL(KIND=8)    ::  wstdlincomf
        REAL(KIND=8)    ::  jlincomf
        REAL(KIND=8)    ::  lincomfstore(mdim)              ! not used
        REAL(KIND=8)    ::  wlincomfstore                   ! not used
        REAL(KIND=8)    ::  wstdlincomfstore                ! not used
        REAL(KIND=8)    ::  jlincomfstore                   ! not used
    END TYPE lcobj
    TYPE   ::  welfareobj                                  ! not used
        REAL(KIND=8)    ::  ba
        REAL(KIND=8)    ::  cs
        REAL(KIND=8)    ::  cs0
    END TYPE welfareobj    
!    TYPE   ::  simdataobj

    TYPE   ::  simobj
        REAL(KIND=8)    ::  chraw(8)=0.d0         ! choice raw count
        INTEGER        ::  chyrraw(8,8)=0        ! choice*year raw count
        INTEGER        ::  chwvraw(12,8)=0       ! choice*wave raw count        
        REAL(KIND=8)    ::  chyrdist(8,8)=0.d0    ! choice distribution by year (each year sum to 100 percent) - generate in final step
        REAL(KIND=8)    ::  achyrdist(8,8)=0.d0   ! ACTUAL choice distribution by year - generate in final step
        REAL(KIND=8)    ::  chwvdist(12,8)=0.d0   ! choice distribution by wave (each year sum to 100 percent) - generate in final step
        REAL(KIND=8)    ::  outyr(8,4)=0.d0       ! participation rates by year
        REAL(KIND=8)    ::  outwv(12,4)=0.d0      ! participation rates by wave        
        REAL(KIND=8)    ::  outwv_p96(12,4)=0.d0
        REAL(KIND=8)    ::  outwv_p923(12,4)=0.d0
        INTEGER        ::  cntwv_p96(12)=0
        INTEGER        ::  cntwv_p923(12)=0
        REAL(KIND=8)    ::  outcome(10)=0.d0        ! overall: 4 participation rates, earnings, utility, 4 benefits
        REAL(KIND=8)    ::   wageidx(33)= (/ (4.5d0+DBLE(iloop-1)*0.5d0 , iloop=1,33 ) /)   ! wage index (histogram) (34 intervals)
        REAL(KIND=8)    ::  wageidx_csq(12)=(/ (1.5D0+DBLE(iloop-1)*0.125d0 , iloop=1,12 ) /) ! wage index (chisq test) (13 intervals)  
        REAL(KIND=8)    ::  wagedist(34)=0.d0       ! wage distribution (histogram)
        REAL(KIND=8)    ::  awagedist(34)=0.d0      ! ACTUAL distribution (histogram)
        REAL(KIND=8)    ::  wagedist_csq(13)=0.d0  ! wage distribution (chisq test)
        REAL(KIND=8)    ::  awagedist_csq(13)=0.d0   ! ACTUAL wage distribution (chisq test)
        REAL(KIND=8)    ::  choice_csq(64)=0.d0 ! choice distribution (year-choice) (chisq test) - generate in final step
        REAL(KIND=8)    ::  achoice_csq(64)=0.d0 ! ACTUAL choice distribution (chisq test) (input data matrix: year*choice)
        INTEGER        ::  numsim=0                ! total number of simulations
        INTEGER        ::  numsimwkr=0             ! total number of simulations (workers only)
        INTEGER        ::  chsample=0              ! ACTUAL total sample
        INTEGER        ::  wkrsample=0             ! ACTUAL worker sample
        REAL(KIND=8)    ::  wagecsq(3)=0.d0         ! wage chi-sq test result
        REAL(KIND=8)    ::  chcsq(3)=0.d0           ! choice-year chi-sq test result
        INTEGER        ::  numsimnonwkr=0
        REAL(KIND=8)    ::  jobprob=0.D0
        REAL(KIND=8)    ::  jobprob_0=0.D0
        REAL(KIND=8)    ::  jobprob_1=0.D0
        REAL(KIND=8)    ::  wpref=0.D0
        REAL(KIND=8)    ::  apref=0.d0
        REAL(KIND=8)    ::  fpref=0.d0
        REAL(KIND=8)    ::  jmean=0.d0
    END TYPE simobj
    TYPE(simobj)   ::  simin
        
END MODULE DATASTORE

module likeparam                                           ! module for likefcn only
    use datastore, only: paranummax
    IMPLICIT NONE
    real    ::  currentmin=0.e0, currentminarg(paranummax)=0.e0
    integer    ::  niter=0    
end module likeparam

MODULE bensim
    USE platform, only: DBL,tol
    CONTAINS
        REAL(KIND=DBL) PURE FUNCTION beitc(earn, arr)
            IMPLICIT NONE
            REAL(KIND=DBL),INTENT(IN)  ::  earn
            REAL(KIND=DBL),INTENT(IN),DIMENSION(:)    ::  arr
            REAL(KIND=DBL)  ::  lev1,lev2,lev3,benmax,rate1,rate2
            lev1=arr(1)
            lev2=arr(2)
            lev3=arr(3)
            benmax=arr(4)
            rate1=arr(5) 
            rate2=arr(6) 
            if (earn<tol) then
                beitc=0.d0
            else if (earn< lev1) then   ! phase-in stage
                beitc=earn * rate1
            else if (earn< lev2) then   ! flat stage
                beitc=benmax
            else                       ! phase-out stage
                beitc=MAX(benmax - rate2*(earn-lev2), 0.d0)
            end if
        END FUNCTION beitc            
        
        REAL(KIND=DBL) PURE FUNCTION bafdc(earn,nlinc,arr)
            use datastore, only: parain
            IMPLICIT NONE
            REAL(KIND=DBL),INTENT(IN) :: earn,nlinc
            REAL(KIND=DBL),INTENT(IN),DIMENSION(:)    ::  arr
            REAL(KIND=DBL)  ::  lev,bengee,benmax,bendol,benpct,ratable     
            REAL(KIND=DBL)  ::  netinc
            lev=arr(1)
            bengee=arr(2)
            benmax=arr(3)
            bendol=arr(4)
            benpct=arr(5)
            ratable=arr(6)
            if (earn+nlinc>lev .or. nlinc>=bengee) then      
                bafdc=0.d0
            else if (earn<bendol+tol) then              
                bafdc= MIN (benmax, ratable*(bengee-nlinc)) 
            else                                         
                netinc= (earn-bendol)*benpct
                bafdc= MIN (benmax, ratable*(bengee-nlinc-netinc))               
                if (bafdc<0.d0) bafdc=0.d0                
            end if
            IF (parain%lcftype==5) bafdc=bafdc*0.7d0
        END FUNCTION bafdc
        
        REAL(KIND=DBL) PURE FUNCTION bfs(earn,nlinc,bafdc,arr)
            USE datastore,only: parain
            IMPLICIT NONE
            REAL(KIND=DBL),INTENT(IN) :: earn,nlinc,bafdc
            REAL(KIND=DBL),INTENT(IN),DIMENSION(:)    ::  arr
            REAL(KIND=DBL)  ::  lev1,lev2,benmax,hded                  
            REAL(KIND=DBL)  ::  netinc            
            lev1=arr(1)
            lev2=arr(2)
            benmax=arr(3)
            hded=arr(4)    
            if (earn+nlinc>lev1) then  
                bfs=0.d0
            else
                netinc= MAX (0.d0, 0.8d0*earn + nlinc + bafdc - 134.d0 - hded)
                if (netinc >lev2) then 
                    bfs=0.d0
                else if (netinc < tol) then
                    bfs = benmax
                else
                    bfs = benmax - parain%fsrate * netinc                   
                    if (bfs<0.d0) bfs=0.d0                    
                end if
            end if
        END FUNCTION bfs
        
        REAL(KIND=DBL) PURE FUNCTION tptax(earn)
            IMPLICIT NONE
            REAL(KIND=DBL),INTENT(IN) :: earn
            if (earn<tol) then
                tptax=0.d0
            else
                tptax=0.0765d0*earn
            end if
        END FUNCTION tptax
        
        REAL(KIND=DBL) PURE FUNCTION tftax(earn,arr)
            use datastore,only: parain
            IMPLICIT NONE            
            REAL(KIND=DBL),INTENT(IN) :: earn
            REAL(KIND=DBL),INTENT(IN),DIMENSION(:)    ::  arr
            REAL(KIND=DBL)  ::  lev,benmax,rate1,rate2,ded
            REAL(KIND=DBL)  ::  netinc
            rate1=arr(1)
            ded=arr(2) 
            if (earn<ded) then 
                tftax=0.d0
            else 
                netinc=earn-ded
                tftax=netinc*rate1
            end if
            tftax=tftax+parain%tprate*earn
        END FUNCTION tftax
        
        REAL(KIND=DBL) PURE FUNCTION sttax(earn,beitc,arr)
            IMPLICIT NONE
            REAL(KIND=DBL),INTENT(IN) :: earn,beitc
            REAL(KIND=DBL),INTENT(IN),DIMENSION(:)    ::  arr     ! there are 3+3*6+3=24 parameters
            REAL(KIND=DBL)  ::  stindic, nbracket, stded, stb(6), str(6), stmb(6)
            REAL(KIND=DBL)  ::  seitc, seitcref, seitcp
            REAL(KIND=DBL)  ::  y,yeitc
            REAL(KIND=DBL)  ::  netinc
            INTEGER       ::   nbracketint
            stindic=arr(1)          ! state income tax indicator
            nbracket=arr(2)         ! number of state income tax brackets
            stded=arr(3)            ! amount of total deductions and exemptions
            stb=arr(4:9)      ! tax brackets
            str=arr(10:15)       ! tax rates
            stmb=arr(16:21)         ! tax maximum "benefit" in each bracket
            seitc=arr(22)           ! state EITC indicator
            seitcref=arr(23)        ! state EITC refundability indicator
            seitcp=arr(24)          ! state EITC as proportion of federal EITC
            nbracketint=INT(nbracket)            
            IF (stindic>tol .AND. earn>stded) THEN     ! state income tax exists AND net income exceed deduction
                netinc = earn-stded
                SELECT CASE (nbracketint)             ! there is a maximum of 6 brackets. If stindic=0 then nbracket=0
                CASE (1)
                    IF (netinc<stb(1)) THEN
                        y=netinc*str(1)                    
                    ELSE 
                        y=stmb(1)
                    END IF
                CASE (2)
                    IF (netinc<stb(1)) THEN
                        y=netinc*str(1)
                    ELSEIF (stb(1)<=netinc<stb(2)) THEN
                        Y=(netinc-stb(1))*str(2) + stmb(1)
                    ELSE
                        y=stmb(2)
                    END IF
                CASE (3)
                    IF (netinc<stb(1)) THEN
                        y=netinc*str(1)
                    ELSEIF (stb(1)<=netinc<stb(2)) THEN
                        Y=(netinc-stb(1))*str(2) + stmb(1)
                    ELSEIF (stb(2)<=netinc<stb(3)) THEN
                        Y=(netinc-stb(2))*str(3) + stmb(2)
                    ELSE
                        y=stmb(3)
                    END IF
                CASE (4)
                    IF (netinc<stb(1)) THEN
                        y=netinc*str(1)
                    ELSEIF (stb(1)<=netinc<stb(2)) THEN
                        Y=(netinc-stb(1))*str(2) + stmb(1)
                    ELSEIF (stb(2)<=netinc<stb(3)) THEN
                        Y=(netinc-stb(2))*str(3) + stmb(2)
                    ELSEIF (stb(3)<=netinc<stb(4)) THEN
                        Y=(netinc-stb(3))*str(4) + stmb(3)                
                    ELSE
                        y=stmb(4)
                    END IF
                CASE (5)
                    IF (netinc<stb(1)) THEN
                        y=netinc*str(1)
                    ELSEIF (stb(1)<=netinc<stb(2)) THEN
                        Y=(netinc-stb(1))*str(2) + stmb(1)
                    ELSEIF (stb(2)<=netinc<stb(3)) THEN
                        Y=(netinc-stb(2))*str(3) + stmb(2)
                    ELSEIF (stb(3)<=netinc<stb(4)) THEN
                        Y=(netinc-stb(3))*str(4) + stmb(3)
                    ELSEIF (stb(4)<=netinc<stb(5)) THEN
                        Y=(netinc-stb(4))*str(5) + stmb(4)
                    ELSE
                        y=stmb(5)
                    END IF
                CASE (6)
                    IF (netinc<stb(1)) THEN
                        y=netinc*str(1)
                    ELSEIF (stb(1)<=netinc<stb(2)) THEN
                        Y=(netinc-stb(1))*str(2) + stmb(1)
                    ELSEIF (stb(2)<=netinc<stb(3)) THEN
                        Y=(netinc-stb(2))*str(3) + stmb(2)
                    ELSEIF (stb(3)<=netinc<stb(4)) THEN
                        Y=(netinc-stb(3))*str(4) + stmb(3)
                    ELSEIF (stb(4)<=netinc<stb(5)) THEN
                        Y=(netinc-stb(4))*str(5) + stmb(4)
                    ELSEIF (stb(5)<=netinc<stb(6)) THEN
                        Y=(netinc-stb(5))*str(6) + stmb(5)
                    ELSE
                        Y=stmb(6)
                    END IF
                END SELECT 
            ELSE
                    y=0.d0
            END IF 
            IF (seitc>tol .AND. seitcref>tol) THEN                        ! state EITC exists and is refundable
                yeitc=beitc*seitcp
            ELSEIF (seitc>tol .AND. seitcref<tol) THEN                    ! state EITC exists and is non-refundable            
                yeitc=beitc*seitcp
                IF (yeitc>y) yeitc=y
            ELSE
                yeitc=0.d0
            END IF 
            sttax=y-yeitc
        END FUNCTION              
END MODULE bensim

MODULE SUBROUTINES
    CONTAINS
    SUBROUTINE complincom(dataobji, parain, lc)
        use mkl95_blas    ! for gemv, DOT
        Use datastore, only: dataobj, paraobj, lcobj
        IMPLICIT NONE
        TYPE(dataobj),INTENT(IN)  ::  dataobji
        TYPE(paraobj),INTENT(IN)  ::  parain
        TYPE(lcobj), INTENT(OUT)  ::  lc
        lc%choice=dataobji%choice       ! not used later?
        lc%wage=dataobji%wage           ! not used later?
        lc%lincom=parain%errmean
        lc%lincom( (/ 2,3,5,7,8 /) ) = lc%lincom( (/ 2,3,5,7,8 /) ) + DOT(parain%pvarwork, dataobji%varwork)
        lc%lincom( (/ 4,5 /) ) = lc%lincom( (/ 4,5 /) ) + DOT(parain%pvarafdc, dataobji%varafdc)
        lc%lincom( (/ 6,7,8 /) ) = lc%lincom( (/ 6,7,8 /) ) + DOT(parain%pvarfs, dataobji%varfs)
        !CALL gemv( parain%pvar, dataobji%var, lc%lincom, beta=1.d0)                          
        lc%wlincom = DOT(parain%pvarwage, dataobji%varwage) + parain%wstat(1)            
        lc%wstdlincom = DOT(parain%pvarwstd, dataobji%varwage) + parain%wstat(2)                
        lc%jlincom = DOT( parain%pvarjob, dataobji%varjob) + parain%jobprobmean
    END SUBROUTINE complincom

    ! Load linear combinations; add factor loadings to obtain factor-adjusted linear combinations
    SUBROUTINE complincomf(lc, parain, iloop)
        Use datastore, only: paraobj, lcobj    
        IMPLICIT NONE
        TYPE(paraobj),INTENT(IN)  ::  parain
        TYPE(lcobj), INTENT(INOUT)  ::  lc
        INTEGER,INTENT(IN)    ::  iloop
        lc%lincomf=lc%lincom
        lc%lincomf( (/ 2,3,5,7,8 /) ) = lc%lincomf( (/ 2,3,5,7,8 /) ) + parain%factorwork(iloop)
        lc%lincomf( (/ 4,5 /) ) = lc%lincomf( (/ 4,5 /) ) + parain%factorafdc(iloop)
        lc%lincomf( (/ 6,7,8 /) ) = lc%lincomf( (/ 6,7,8 /) ) + parain%factorfs(iloop)
        lc%wlincomf = lc%wlincom + parain%factorw(iloop)
        lc%wstdlincomf = lc%wstdlincom
        lc%jlincomf = lc%jlincom + parain%factorj(iloop)        
    END SUBROUTINE complincomf               
    
    ! Load choice, wage, explanatory variables from data; multiply by parameters to obtain linear combinations
    ! When multiplying, omit lagged dependent variables (state variables). For dynprog only.
    SUBROUTINE complincomdyn(dataobji, parain, lc)
        use mkl95_blas    ! for gemv, DOT
        Use datastore, only: dataobj, paraobj, lcobj
        IMPLICIT NONE
        TYPE(dataobj),INTENT(IN)  ::  dataobji
        TYPE(paraobj),INTENT(IN)  ::  parain
        TYPE(lcobj), INTENT(OUT)  ::  lc
        lc%choice=dataobji%choice   ! not used later?
        lc%wage=dataobji%wage       ! not used later?
        lc%lincom=parain%errmean
        lc%lincom( (/ 2,3,5,7,8 /) ) = lc%lincom( (/ 2,3,5,7,8 /) ) + DOT(parain%pvarwork(2:parain%varworkdim), dataobji%varwork(2:parain%varworkdim))
        lc%lincom( (/ 4,5 /) ) = lc%lincom( (/ 4,5 /) ) + DOT(parain%pvarafdc(2:parain%varafdcdim), dataobji%varafdc(2:parain%varafdcdim))
        lc%lincom( (/ 6,7,8 /) ) = lc%lincom( (/ 6,7,8 /) ) + DOT(parain%pvarfs(2:parain%varfsdim), dataobji%varfs(2:parain%varfsdim))
        !CALL gemv( parain%pvar, dataobji%var, lc%lincom, beta=1.d0)                          
        lc%wlincom = DOT(parain%pvarwage(2:parain%varwagedim), dataobji%varwage(2:parain%varwagedim)) + parain%wstat(1)                !+0.1d0
        lc%wstdlincom = DOT(parain%pvarwstd, dataobji%varwage) + parain%wstat(2)                
        lc%jlincom = DOT( parain%pvarjob(2:parain%varjobdim), dataobji%varjob(2:parain%varjobdim)) + parain%jobprobmean
    END SUBROUTINE complincomdyn
    
    ! Load linear combinations; add factor loadings to obtain factor-adjusted linear combinations
    ! When adding, lagged dependent variables (state variables) are added as well. For dynprog only.
    SUBROUTINE complincomfdyn(lc, parain, iloop, lwork, lafdc, lfs)
        Use datastore, only: paraobj, lcobj    
        IMPLICIT NONE
        TYPE(paraobj),INTENT(IN)  ::  parain
        TYPE(lcobj), INTENT(INOUT)  ::  lc
        INTEGER,INTENT(IN)    ::  iloop
        REAL(KIND=8),INTENT(IN)    ::  lwork, lafdc, lfs
        lc%lincomf=lc%lincom
        lc%lincomf( (/ 2,3,5,7,8 /) ) = lc%lincomf( (/ 2,3,5,7,8 /) ) + parain%factorwork(iloop) + parain%pvarwork(1)*lwork
        lc%lincomf( (/ 4,5 /) ) = lc%lincomf( (/ 4,5 /) ) + parain%factorafdc(iloop) + parain%pvarafdc(1)*lafdc
        lc%lincomf( (/ 6,7,8 /) ) = lc%lincomf( (/ 6,7,8 /) ) + parain%factorfs(iloop) + parain%pvarfs(1)*lfs
        lc%wlincomf = lc%wlincom + parain%factorw(iloop) + parain%pvarwage(1)*lwork*0.1d0
        lc%wstdlincomf = lc%wstdlincom
        lc%jlincomf = lc%jlincom + parain%factorj(iloop)  + parain%pvarjob(1)*lwork*0.1d0
    END SUBROUTINE complincomfdyn            

    SUBROUTINE initerr
        use datastore, only: stream
        use mkl_vsl
        use mkl_vsl_type
        IMPLICIT NONE
        INTEGER    ::  iostat, iloop             
        do iloop=1,8
            iostat=vslnewstream( stream(iloop), VSL_BRNG_MT2203,  700+iloop )       ! set seed for random stream array
            if (iostat /= vsl_error_ok) print *, 'error new stream in INITERR'            
        end do
    END SUBROUTINE initerr    
    
    SUBROUTINE lnden(out, in, mean, recstd2, lgstd)
        IMPLICIT NONE
        REAL(KIND=8), PARAMETER    ::  constant=-0.918938533204673d0
        REAL(KIND=8),INTENT(OUT)   ::  out
        REAL(KIND=8),INTENT(IN)    ::  in, mean, recstd2, lgstd ! recstd2=1/sigma**2; lgstd=log(sigma)
        out=constant-lgstd-0.5d0*recstd2*(in-mean)**2
    END SUBROUTINE lnden       
    SUBROUTINE lndeneasy(out, in, mean, sigma)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(OUT)   ::  out
        REAL(KIND=8),INTENT(IN)    ::  in, mean, sigma
        CALL lnden(out, in, mean, 1.d0/(sigma**2), LOG(sigma) )
    END SUBROUTINE lndeneasy    
    
    ! Compute income for each choice (given wage and data in period t)
    SUBROUTINE geninc(income,wage,dataobji)
        USE mkl95_blas      ! FUNCTION DOT
        USE bensim          ! LOAD BENEFIT FORMULAS
        USE datastore, only: dataobj, parain
        IMPLICIT NONE
        TYPE(dataobj),INTENT(IN)  ::  dataobji
        REAL(KIND=8),INTENT(IN)    ::  wage
        REAL(KIND=8),INTENT(OUT)   ::  income(:)           
        REAL(KIND=8)    ::  wage2, afdc1, afdc2, tax2, tax3              
        REAL(KIND=8)    ::  beitc1,beitc2    
        income(1)=0.d0  
        
        ! COMPUTE WELFARE BENEFIT UNDER NO-WORK AND PART-TIME WORK
        IF (dataobji%hit==0) THEN
            afdc1=bafdc(0.d0, 0.d0, dataobji%afdc) - DOT( parain%psanction, dataobji%sanction)
            afdc2=bafdc(wage, 0.d0, dataobji%afdc)        
        ELSEIF (dataobji%hit==1 .AND. dataobji%tlred==0) THEN
            afdc1=-999999.d0
            afdc2=-999999.d0            
        ELSEIF (dataobji%hit==1 .AND. dataobji%tlred==1) THEN
            afdc1=bafdc(0.d0, 0.d0, dataobji%afdc) * dataobji%rfoklt18/(dataobji%rfoklt18+1.d0) - DOT( parain%psanction, dataobji%sanction)
            afdc2=bafdc(wage, 0.d0, dataobji%afdc) * dataobji%rfoklt18/(dataobji%rfoklt18+1.d0)
        ELSE
            PRINT *, 'ERROR IN GENINC'
            STOP           
        END IF
        
        ! NO JOB OFFER: NON-WORK CHOICES ONLY
        IF (wage<0.01d0) THEN
            income ( (/ 2,3,5,7,8 /) ) = -999999.d0
            income(4) = afdc1 +   bfs(0.d0,0.d0,afdc1,dataobji%fs)
            income(6) = bfs(0.d0,0.d0,0.d0,dataobji%fs)
        ! WITH JOB OFFER: ALL CHOICES
        ELSE        
            wage2=wage*2.d0                                                 ! full-time gross earnings (work choices only)
            beitc1=beitc(wage, dataobji%eitc)
            beitc2=beitc(wage2, dataobji%eitc)
            tax2=beitc1 - tftax(wage, dataobji%ftax) -sttax(wage, beitc1,dataobji%sttax)   ! part-time eitc and tax   (work choices only)
            tax3=beitc2 - tftax(wage2, dataobji%ftax)-sttax(wage2, beitc2,dataobji%sttax)  ! full-time eitc and tax   (work choices only)
            income(2)=wage+tax2
            income(3)=wage2+tax3                    
            income(4)=afdc1 +   bfs(0.d0,0.d0,afdc1,dataobji%fs)
            income(5)=wage  +   afdc2 +   bfs(wage,0.d0,afdc2,dataobji%fs)    + tax2
            income(6)=bfs(0.d0,0.d0,0.d0,dataobji%fs)
            income(7)=wage  +   bfs(wage,0.d0,0.d0,dataobji%fs)     + tax2
            income(8)=wage2 +   bfs(wage2,0.d0,0.d0,dataobji%fs)    + tax3
        END IF        
        IF (dataobji%rfoklt18 < 0.01d0 ) income( (/4,5,6,7,8/) ) = -999999.d0                
    END SUBROUTINE geninc            
    
    ! Simulate income and benefit amount
    SUBROUTINE genincsim(income,wage,dataobji, ben, ind )
        USE mkl95_blas      ! FUNCTION DOT
        USE bensim
        USE datastore, only: dataobj, parain
        IMPLICIT NONE
        TYPE(dataobj),INTENT(IN)  ::  dataobji
        REAL(KIND=8),INTENT(IN)    ::  wage
        REAL(KIND=8),INTENT(OUT)   ::  income(:)           
        REAL(KIND=8)    ::  wage2, afdc1, afdc2, tax2, tax3   
        REAL(KIND=8),OPTIONAL  ::  ben(8,4)        ! BENEFIT AMOUNT MATRIX (COLUMNS: 1 WELFARE, 2 FS, 3 EITC, 4 TAX)
        INTEGER,INTENT(IN), optional    ::  ind  ! DEFAULT=4
        REAL(KIND=8)    ::  wagearr(8), eitc2, eitc3, ftax2, ftax3
        REAL(KIND=8)    ::  stax2, stax3        
        income(1)=0.d0  ! no work and no programs - set income=0
                
        ! COMPUTE WELFARE BENEFIT UNDER NO-WORK AND PART-TIME WORK
        IF (dataobji%hit==0) THEN
            afdc1=bafdc(0.d0, 0.d0, dataobji%afdc) - DOT( parain%psanction, dataobji%sanction)
            afdc2=bafdc(wage, 0.d0, dataobji%afdc)        
        ELSEIF (dataobji%hit==1 .AND. dataobji%tlred==0) THEN
            afdc1=-999999.d0
            afdc2=-999999.d0            
        ELSEIF (dataobji%hit==1 .AND. dataobji%tlred==1) THEN
            afdc1=bafdc(0.d0, 0.d0, dataobji%afdc) * dataobji%rfoklt18/(dataobji%rfoklt18+1.d0) - DOT( parain%psanction, dataobji%sanction)
            afdc2=bafdc(wage, 0.d0, dataobji%afdc) * dataobji%rfoklt18/(dataobji%rfoklt18+1.d0)
        ELSE
            PRINT *, 'ERROR IN GENINC'
            STOP           
        END IF
        wage2=wage*2.d0              
        eitc2=beitc(wage, dataobji%eitc)
        eitc3=beitc(wage2, dataobji%eitc)
        ftax2= tftax(wage, dataobji%ftax)
        ftax3= tftax(wage2, dataobji%ftax)    
        stax2= sttax(wage, eitc2, dataobji%sttax)
        stax3= sttax(wage2, eitc3, dataobji%sttax)
        ben=0.d0
        SELECT CASE (ind)
        CASE (4)       ! EITC, TAX, WELFARE AND FOOD STAMPS (DEFAULT=4)  
            IF ( parain%lcftype==8) THEN
                IF (dataobji%rfoklt18==0 ) THEN
                    ben( (/2,5,7/) ,3)= eitc2
                    ben( (/3,8/) ,3)= eitc3
                ELSEIF (dataobji%rfoklt18==1 ) THEN
                    ben( (/2,5,7/) ,3)= eitc2 + MAX(100.d0-wage*0.07d0,0.d0)
                    ben( (/3,8/) ,3)= eitc3 + MAX(100.d0-wage2*0.07d0,0.d0)                        
                ELSE
                    ben( (/2,5,7/) ,3)= eitc2 + MAX(200.d0-wage*0.07d0,0.d0)
                    ben( (/3,8/) ,3)= eitc3 + MAX(200.d0-wage2*0.07d0,0.d0)                       
                END IF        
            ELSEIF ( parain%lcftype==14) THEN     ! WORK SUBSIDY TYPE 2
                IF (dataobji%rfoklt18==0 ) THEN
                    ben( (/2,5,7/) ,3)= eitc2
                    ben( (/3,8/) ,3)= eitc3
                ELSEIF (dataobji%rfoklt18==1 ) THEN
                    ben( (/2,5,7/) ,3)= eitc2 + MAX(200.d0-wage*0.16d0,0.d0)
                    ben( (/3,8/) ,3)= eitc3 + MAX(200.d0-wage2*0.16d0,0.d0)                        
                ELSE
                    ben( (/2,5,7/) ,3)= eitc2 + MAX(300.d0-wage*0.21d0,0.d0)
                    ben( (/3,8/) ,3)= eitc3 + MAX(300.d0-wage2*0.21d0,0.d0)                       
                END IF                        
            ELSE            
                ben( (/2,5,7/) ,3)= eitc2
                ben( (/3,8/) ,3)= eitc3
            END IF
            ben( (/2,5,7/) ,4)= -ftax2 -stax2
            ben( (/3,8/) ,4)= -ftax3   -stax3
            ben(4,1)=afdc1
            ben(5,1)=afdc2             
            ben(4,2)=bfs(0.d0,0.d0,afdc1,dataobji%fs)
            ben(5,2)=bfs(wage,0.d0,afdc2,dataobji%fs)
            ben(6,2)=bfs(0.d0,0.d0,0.d0,dataobji%fs)
            ben(7,2)=bfs(wage,0.d0,0.d0,dataobji%fs)
            ben(8,2)=bfs(wage2,0.d0,0.d0,dataobji%fs)            
        CASE (0)       ! NO PROGRAMS AT ALL
        CASE (1)       ! EITC ONLY
            ben( (/2,5,7/) ,3)= eitc2
            ben( (/3,8/) ,3)= eitc3            
        CASE (2)       ! EITC AND TAX ONLY
            ben( (/2,5,7/) ,3)= eitc2
            ben( (/3,8/) ,3)= eitc3
            ben( (/2,5,7/) ,4)= -ftax2 -stax2
            ben( (/3,8/) ,4)= -ftax3   -stax3
        CASE (3)       ! EITC, TAX AND FOOD STAMPS
            ben( (/2,5,7/) ,3)= eitc2
            ben( (/3,8/) ,3)= eitc3
            ben( (/2,5,7/) ,4)= -ftax2 -stax2
            ben( (/3,8/) ,4)= -ftax3   -stax3
            ben(4,2)=bfs(0.d0,0.d0,0.d0,dataobji%fs)
            ben(5,2)=bfs(wage,0.d0,0.d0,dataobji%fs)
            ben(6,2)=bfs(0.d0,0.d0,0.d0,dataobji%fs)
            ben(7,2)=bfs(wage,0.d0,0.d0,dataobji%fs)
            ben(8,2)=bfs(wage2,0.d0,0.d0,dataobji%fs)                           
        CASE (5)       ! TAX, WELFARE AND FOOD STAMPS
            ben( (/2,5,7/) ,4)= -ftax2 -stax2
            ben( (/3,8/) ,4)= -ftax3   -stax3
            ben(4,1)=afdc1
            ben(5,1)=afdc2             
            ben(4,2)=bfs(0.d0,0.d0,afdc1,dataobji%fs)
            ben(5,2)=bfs(wage,0.d0,afdc2,dataobji%fs)
            ben(6,2)=bfs(0.d0,0.d0,0.d0,dataobji%fs)
            ben(7,2)=bfs(wage,0.d0,0.d0,dataobji%fs)
            ben(8,2)=bfs(wage2,0.d0,0.d0,dataobji%fs)                  
        END SELECT           
        income = (/ 0.d0, wage, wage2, 0.d0, wage, 0.d0, wage, wage2 /) + ben(:,1) + ben(:,2) + ben(:,3) + ben(:,4)        
        IF (dataobji%rfoklt18 < 0.01d0 ) income( (/4,5,6,7,8/) ) = -999999.d0      
        IF (wage<0.01d0) income( (/2,3,5,7,8/) ) = -999999.d0
    END SUBROUTINE genincsim        
        
    ! Remap parameters from array X to relevant elements in parain
    SUBROUTINE remapparam(X,parain)
        USE datastore, only: paraobj, paranum
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  X(:)
        TYPE(paraobj),INTENT(INOUT)   ::  parain        
        INTEGER    ::  index1,index2
        REAL(KIND=8)   ::  temparr(1)
        temparr=0.d0
        parain%errvar=EXP(X(1))/1.28d0              ! EXP(X(1)) is sigma_c*pi/sqrt(6). Normalize to sigma_c in model for emax calculation
        ! X(2) phi_H; X(3) phi_3; X(4) phi_AF; X(5) phi_5; X(6) phi_F; X(7) phi_7; X(8) phi_8
        parain%errmean(1)=0.d0
        parain%errmean(2)=X(2)
        parain%errmean(3)=X(3)+X(2)
        parain%errmean(4)=X(4)
        parain%errmean(5)=X(5)+X(2)+X(4)
        parain%errmean(6)=X(6)
        parain%errmean(7)=X(7)+X(2)+X(6)
        parain%errmean(8)=X(8)+X(2)+X(3)+X(6)
        parain%wstat(1)=X(9)
        parain%wstat(2)=EXP(X(10))
        index1=10
        IF (parain%cjob==1) THEN
            CALL remap(index1,1,X,temparr)
            parain%jobprobmean=temparr(1)
        END IF
        ! Recursively update the parameter count and save elements of X into parain
        IF (parain%cbig==1) THEN
            CALL remap(index1,parain%npwork,X,parain%pvarwork)
            CALL remap(index1,parain%npafdc,X,parain%pvarafdc)
            CALL remap(index1,parain%npfs,X,parain%pvarfs)
            CALL remap(index1,parain%npwage,X,parain%pvarwage)
            IF (parain%cjob==1)         CALL remap(index1,parain%npjob,X,parain%pvarjob)
        END IF
        IF (parain%csanction==1)    CALL remap(index1,parain%npsanction,X,parain%psanction)
        IF (parain%ctype==1) THEN      ! only applicable if at least 2 types            
            parain%deltaindex = index1+1                  ! save index for delta method calculation
            index2=index1 + parain%nptype-1
            call logit( parain%typeprob(1:parain%nptype), (/ 0.d0,  X(index1+1:index2) /) )              
            index1=index2                    
            CALL remap(index1,parain%nptype-1,X,parain%factorwork(2:parain%typenum))
            CALL remap(index1,parain%nptype-1,X,parain%factorafdc(2:parain%typenum))             
            CALL remap(index1,parain%nptype-1,X,parain%factorfs(2:parain%typenum))
            CALL remap(index1,parain%nptype-1,X,parain%factorw(2:parain%typenum))            
            IF (parain%cjob==1) CALL remap(index1,parain%nptype-1,X,parain%factorj(2:parain%typenum))
        END IF
        IF (parain%cdisc==1) THEN
            CALL remap(index1,1,X,temparr)
            parain%discount = blogit ( temparr(1)+3.d0 )            
        END IF
        IF (parain%cquad==1) THEN
            CALL remap(index1,1,X,temparr)
            parain%quadratic= temparr(1) * 0.0001d0                 ! normalize quadratic term by a factor of 10000
        END IF
        IF (index1 .NE. paranum) THEN
            PRINT *, 'MISMATCH BETWEEN PARANUM (', PARANUM, ') AND INDEX (', INDEX1, ') IN REMAPPARAM'
            STOP
        END IF
        CONTAINS
            SUBROUTINE remap(index,nparam,X,Y)
                IMPLICIT NONE
                INTEGER,INTENT(INOUT) ::  index
                INTEGER,INTENT(IN)    ::  nparam            
                REAL(KIND=8),DIMENSION(:),INTENT(IN)  ::  X                        
                REAL(KIND=8),DIMENSION(:),INTENT(INOUT)  ::  Y
                Y(1:nparam) = X(index+1:index+nparam)
                index=index+nparam
            END SUBROUTINE remap        
    END SUBROUTINE remapparam
        
    SUBROUTINE LIKEFCN(N,X,F)
        use datastore, only: dataobj, paraobj, gdatain, parain, & 
             TID, OMP_GET_THREAD_NUM, CHUNKSIZE, NTHREAD, &         ! parallel computing parameters
             farr
        use likeparam                           ! load specific monitoring global variables for likelihood estimation
        use mkl95_blas          ! function NRM2
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  N           ! number of parameters
        REAL(KIND=8),INTENT(IN)    ::  X(N)    ! parameters
        REAL(KIND=8),INTENT(OUT)   ::  F       ! likelihood
        TYPE(dataobj)  ::  datain        
        INTEGER    ::  iloop        
        farr=0.d0
        CALL remapparam(X,parain)
        CALL OMP_SET_NUM_THREADS(nthread)                                              ! external function, set total number of threads
        TID = OMP_GET_THREAD_NUM()                                                      ! obtain thread number (private to each thread)
        do iloop=1,parain%groupsize                                                    ! loop variable is private by default
                IF (parain%cdynprog==1) CALL dynprog (gdatain(iloop), parain)          
                CALL compliketype(farr(iloop), gdatain(iloop), parain)            
        end do     
        F=-SUM(farr(1:parain%groupsize))        
        IF ( currentmin==0.D0 .OR. ( (F<currentmin) .AND. nrm2(X-currentminarg(1:parain%paranum))>1.D-3 ) ) then
            currentmin=F   
            currentminarg(1:parain%paranum)=X    
            NITER=NITER+1
            write (1,*) 'iterate # and likelihood=', NITER, F
            PRINT *, 'iterate # and likelihood=', NITER, F                        
            write(1,'(100(5(f13.7, 1x), /))') (x(iloop), iloop=1,N)   
            if (parain%dirsim==1) write (1,'(f13.7)') farr(1:parain%groupsize)            
        END IF
    END SUBROUTINE LIKEFCN    
     
    SUBROUTINE COMPLIKEtype(out, gdataobji, parain)
        use anordf_int          ! computes normal cdf
        use datastore, only: mdim, dataobj, paraobj, gdataobj, lcobj
        IMPLICIT NONE                
        TYPE(gdataobj),INTENT(INOUT)  ::  gdataobji  
        TYPE(paraobj),INTENT(IN)  ::  parain                     
        REAL(KIND=8), INTENT(OUT)  ::  out
        TYPE(lcobj) :: lc
        INTEGER    ::  iloop, kloop, jloop
        REAL(kind=8)  ::  jobprob, factorval, choicesum, wlnden, tempin(parain%ghq%npoint), temparr(parain%ghq%npoint, parain%maxgsize),  temp(parain%typenum)
        REAL(KIND=8),DIMENSION(parain%mdim)    ::  income, valuearr, evvalue, kernel
        tempin=0.d0
        temparr=0.d0
        temp=0.d0
        income=0.d0
        valuearr=0.d0       
        evvalue=0.d0 
        kernel=0.d0          
        do kloop=1,gdataobji%gsize    
            CALL complincom(gdataobji%datain(kloop), parain, lc)                
            do iloop=1, parain%nptype
                CALL complincomf(lc, parain, iloop)
                jobprob = blogit( lc%jlincomf )      
                IF (parain%cdynprog==1) CALL compev(evvalue, gdataobji, parain, iloop, kloop)                                      
                IF (lc%wage .LT. 1.d-4) THEN    ! non-worker
                    do jloop=1,parain%ghq%npoint
                        factorval=MAX( EXP(parain%ghq%absci(jloop) * lc%wstdlincomf + lc%wlincomf ), gdataobji%datain(kloop)%minwage)
                        CALL geninc(income,factorval,gdataobji%datain(kloop))
                        valuearr=(lc%lincomf + income + parain%quadratic*income**2.d0 )/(parain%errvar) + evvalue           ! add quadratic term
                        CALL logit(kernel,valuearr)             
                        tempin(jloop)=kernel(lc%choice)
                    end do                    
                    CALL geninc(income,0.d0,gdataobji%datain(kloop))      ! no offer
                    valuearr=(lc%lincomf + income + parain%quadratic*income**2.d0 )/(parain%errvar)  +evvalue               ! add quadratic term (corrected)
                    CALL logit(kernel,valuearr)             
                    temparr(iloop, kloop)=SUM(tempin * parain%ghq%weight) * jobprob + kernel(lc%choice) * (1.d0-jobprob)                                        
                ELSE
                    CALL geninc(income,lc%wage ,gdataobji%datain(kloop))
                    valuearr=(lc%lincomf + income + parain%quadratic*income**2.d0 )/(parain%errvar) + evvalue               ! add quadratic term
                    CALL logit(kernel,valuearr)                       
                    IF (ABS(lc%wage - gdataobji%datain(kloop)%minwage ) < 5.d0) THEN             
                        wlnden = anordf(   ( LOG(lc%wage) - lc%wlincomf )/ lc%wstdlincomf  )
                        temparr(iloop, kloop) = kernel(lc%choice)*wlnden * jobprob
                    ELSE
                        CALL lndeneasy(wlnden, LOG(lc%wage), lc%wlincomf , lc%wstdlincomf)
                        temparr(iloop, kloop) = kernel(lc%choice)*EXP(wlnden) * jobprob           
                    END IF
                END IF
            end do            
        end do
        do iloop=1, parain%nptype
            temp(iloop)=PRODUCT(temparr(iloop,1:gdataobji%gsize ))        
        end do            
        choicesum=SUM(temp(1:parain%nptype) * parain%typeprob(1:parain%nptype) )        
        out=MAX(LOG(choicesum), -50.d0)  
     END SUBROUTINE COMPLIKEtype       
     
        SUBROUTINE setevtp1 (dynindic, iloop, kloop, gdataobji, discount, indicator, arrayin, evtp1)
        use datastore, only: gdataobj, mdim    
        IMPLICIT NONE
        TYPE(gdataobj),INTENT(IN)  ::  gdataobji    
        REAL(KIND=8),INTENT(IN)  ::  discount       
        INTEGER,INTENT(IN)    ::  dynindic,iloop,kloop, indicator, arrayin(mdim)
        REAL(KIND=8),INTENT(OUT)   ::  evtp1(mdim)
        INTEGER                    ::  jloop
        INTEGER,DIMENSION(mdim)  ::  workarr,afdcarr,fsarr
        IF (dynindic==1) THEN
            workarr= (/1,2,2,1,2,1,2,2/)
            afdcarr= (/1,1,1,2,2,1,1,1/)
            fsarr  = (/1,1,1,2,2,2,2,2/)
        ELSE
            workarr= (/1,1,1,1,1,1,1,1/)
            afdcarr= workarr
            fsarr  = workarr
        END IF            
        IF (indicator==0) THEN
            evtp1 = discount * (/ (gdataobji%evntl(iloop,kloop+1,workarr(jloop),afdcarr(jloop),fsarr(jloop)),jloop=1,mdim) /)
        ELSEIF (indicator==1) THEN
            evtp1 = discount * (/ (gdataobji%ev(iloop, kloop+1, arrayin(jloop), workarr(jloop), afdcarr(jloop), fsarr(jloop)), jloop=1,mdim) /)
        ELSEIF (indicator==2) THEN
            evtp1 = discount * (/ (gdataobji%evhit(iloop, kloop+1, arrayin(jloop), workarr(jloop), afdcarr(jloop), fsarr(jloop)), jloop=1,mdim) /)
        ELSE
            PRINT *, 'INDICATOR ERROR IN SUBROUTINE SETEVTP1'
            STOP
        END IF
    END SUBROUTINE setevtp1
                 
     SUBROUTINE dynprog(gdataobji, parain)
        !use mkl_vml
        use mkl95_blas     
        use datastore, only: paraobj, gdataobj, lcobj
        IMPLICIT NONE                
        TYPE(gdataobj),INTENT(INOUT)  ::  gdataobji
        TYPE(paraobj),INTENT(IN)  ::  parain    
        TYPE(lcobj) :: lc
        INTEGER    ::  iloop, kloop, jloop, mloop, workloop, afdcloop, fsloop, nloop
        !INTEGER    ::  iostat
        REAL(KIND=8)    ::  factorval, OUT
        REAL(KIND=8),DIMENSION(parain%mdim)    ::  evtemp, income
        gdataobji%ev=0.d0        
        gdataobji%evhit=0.d0
        gdataobji%evntl=0.d0    ! 20111224       
        gdataobji%evdatain=gdataobji%datain( 1 )                    
        CALL compevbase(parain%dpsdep,parain,lc,gdataobji)         
        do iloop=1,parain%nptype
            do kloop=gdataobji%horizon-1, 1, -1     
                CALL setevtp1 (parain%dpsdep,iloop, kloop, gdataobji, parain%discount, 0, (/0,0,0,0,0,0,0,0/) , evtemp)  
                do workloop=1,parain%dpsdep+1
                do afdcloop=1,parain%dpsdep+1
                do fsloop=1,parain%dpsdep+1
                    IF (.NOT. (afdcloop==2 .AND. fsloop==1) ) THEN
                    CALL evfunction(iloop, 1, workloop, afdcloop, fsloop, parain, gdataobji, evtemp, out)
                    gdataobji%evntl(iloop, kloop, workloop, afdcloop, fsloop) = out
                    END IF
                end do
                end do
                end do                      
            end do
        end do                    

        tlcompute: IF (gdataobji%tlcompute==1) THEN
        gdataobji%evdatain=gdataobji%datain(gdataobji%tlstart)             
        CALL compevbase(parain%dpsdep,parain,lc,gdataobji)                
        do iloop=1,parain%nptype      
        do kloop=gdataobji%tlhor-1, 1, -1         
            do mloop=gdataobji%tllen, 1, -1       
                IF (mloop==gdataobji%tllen .AND. gdataobji%tlnlife==1) THEN 
                    CALL setevtp1 (parain%dpsdep,iloop, kloop, gdataobji, parain%discount, 2, (/1,1,1,1,1,1,1,1/) , evtemp)  
                ELSE                                                        
                    CALL setevtp1 (parain%dpsdep,iloop, kloop, gdataobji, parain%discount, 1, &
                        (/mloop,mloop,mloop,MIN(mloop+1, gdataobji%tllen),MIN(mloop+1, gdataobji%tllen),mloop,mloop,mloop/) , evtemp)  
                END IF
                do workloop=1,parain%dpsdep+1
                do afdcloop=1,parain%dpsdep+1
                do fsloop=1,parain%dpsdep+1                                                    
                    IF (.NOT. ( afdcloop==2 .AND. fsloop==1) ) THEN
                    CALL evfunction(iloop, 1+INT( indic(mloop==gdataobji%tllen) ), workloop, afdcloop, fsloop, parain, gdataobji, evtemp, out)
                    gdataobji%ev(iloop, kloop, mloop, workloop, afdcloop, fsloop) = out
                    END IF 
                end do
                end do
                end do
            end do          
            
            ! compute evhit (nonlifetime limits)
            tlnlife: IF (gdataobji%tlnlife==1) THEN                            
            do nloop=gdataobji%tlnlifebase, 1, -1
                if (nloop==gdataobji%tlnlifebase) then                         
                    CALL setevtp1 (parain%dpsdep,iloop, kloop, gdataobji, parain%discount, 1, (/1,1,1,1,1,1,1,1/) , evtemp)  
                else                            
                    CALL setevtp1 (parain%dpsdep,iloop, kloop, gdataobji, parain%discount, 2, &
                        (/nloop+1,nloop+1,nloop+1,nloop+1,nloop+1,nloop+1,nloop+1,nloop+1/) , evtemp)
                end if   
                do workloop=1,parain%dpsdep+1
                do afdcloop=1,parain%dpsdep+1
                do fsloop=1,parain%dpsdep+1                                                    
                    IF (.NOT. (afdcloop==2 .AND. fsloop==1) ) THEN
                    CALL evfunction(iloop, 2, workloop, afdcloop, fsloop, parain, gdataobji, evtemp, out) 
                    gdataobji%evhit(iloop, kloop, nloop, workloop, afdcloop, fsloop) = out
                    END IF 
                end do
                end do
                end do                                  
            end do
            END IF tlnlife                                                       
        end do
        end do                
        END IF tlcompute          
        CONTAINS
            PURE SUBROUTINE evfunction(iloop, hit, workloop, afdcloop, fsloop, parain, gdataobji, evtemp, out)            
                use datastore, only: paraobj, gdataobj, mdim
                IMPLICIT NONE
                TYPE(gdataobj),INTENT(IN)  ::  gdataobji
                TYPE(paraobj),INTENT(IN)  ::  parain 
                INTEGER,INTENT(IN)    ::  iloop,hit,workloop,afdcloop,fsloop
                REAL(KIND=8),INTENT(IN)    ::  evtemp(mdim)
                REAL(KIND=8),INTENT(OUT)   ::  out
                REAL(KIND=8)    ::  jobprob, valuearr(mdim), tempin(parain%ghq%npointdyn), evnoffer
                INTEGER    ::  jloop
                jobprob = gdataobji%evjobprob(iloop, workloop, afdcloop, fsloop)
                do jloop=1,parain%ghq%npointdyn
                    valuearr=gdataobji%evbase(iloop, :, jloop, hit, workloop, afdcloop, fsloop ) + evtemp
                    tempin(jloop)     =parain%ghq%euler + MAXVAL(valuearr) + LOG( SUM( EXP(valuearr - MAXVAL(valuearr) ) ) ) ! avoid overflow
                end do                    
                valuearr=gdataobji%evbaseno(iloop, :, hit, workloop, afdcloop, fsloop ) + evtemp
                evnoffer=parain%ghq%euler + MAXVAL(valuearr) + LOG( SUM( EXP(valuearr - MAXVAL(valuearr) ) ) )  ! avoid overflow
                out = ( SUM(tempin * parain%ghq%weightdyn) * jobprob + evnoffer * (1.d0-jobprob) )
            END SUBROUTINE evfunction
             
            SUBROUTINE compevbase(dynindic,parain,lc,gdataobji)
                USE datastore, only: gdataobj,paraobj, lcobj
                IMPLICIT NONE
                INTEGER,INTENT(IN)        ::  dynindic        
                TYPE(paraobj),INTENT(IN)  ::  parain
                TYPE(lcobj),INTENT(INOUT)    ::  lc           
                TYPE(gdataobj),INTENT(INOUT)  ::  gdataobji   
                INTEGER                    ::  iloop,workloop,afdcloop,fsloop,mloop,jloop
                REAL(KIND=8)                ::  factorval,income(parain%mdim)
                gdataobji%evbase=0.d0   
                gdataobji%evbaseno=0.d0     
                gdataobji%evjobprob=0.d0            
                IF (dynindic==1) THEN
                    CALL complincomdyn(gdataobji%evdatain, parain, lc)                
                ELSE        
                    CALL complincom(gdataobji%evdatain, parain, lc)                
                END IF            
                do iloop=1,parain%nptype
                    do workloop=1,dynindic+1
                    do afdcloop=1,dynindic+1
                    do fsloop=1,dynindic+1
                        lim_state: IF (.NOT.(afdcloop==2 .AND. fsloop==1)) THEN                                
                        IF (dynindic==1) THEN
                            CALL complincomfdyn(lc, parain, iloop, indic(workloop==2), indic(afdcloop==2), indic(fsloop==2) )
                        ELSE
                            CALL complincomf(lc, parain, iloop)    
                        END IF                                
                        IF (parain%jelast==0) THEN
                            gdataobji%evjobprob(iloop, workloop, afdcloop, fsloop) = blogit( lc%jlincomf)           ! store evjobprob
                        ELSE
                            gdataobji%evjobprob(iloop, workloop, afdcloop, fsloop) = MIN(blogit( lc%jlincomf) *1.1D0,1.D0)           ! store evjobprob
                        END IF
                        do mloop=1,2
                            gdataobji%evdatain%hit=mloop-1     
                            do jloop=1,parain%ghq%npointdyn
                                IF (parain%welast==0) THEN
                                    factorval=MAX( EXP(parain%ghq%abscidyn(jloop) * lc%wstdlincomf + lc%wlincomf), gdataobji%evdatain%minwage)
                                ELSE
                                    factorval=MAX( EXP(parain%ghq%abscidyn(jloop) * lc%wstdlincomf + lc%wlincomf), gdataobji%evdatain%minwage) *1.1D0
                                END IF
                                CALL geninc(income,factorval,gdataobji%evdatain)                
                                gdataobji%evbase(iloop,:,jloop,mloop,workloop,afdcloop,fsloop)=(lc%lincomf+income+parain%quadratic*income**2.d0)/(parain%errvar)!quad(corr)
                            end do                                            
                            CALL geninc(income,0.d0,gdataobji%evdatain)      ! no offer
                            gdataobji%evbaseno(iloop, :, mloop, workloop, afdcloop, fsloop)=(lc%lincomf + income+parain%quadratic*income**2.d0)/(parain%errvar)!quad(corr)
                        end do
                        END IF lim_state
                    end do
                    end do
                    end do
                end do        
            END SUBROUTINE compevbase       
    END SUBROUTINE dynprog
       
    SUBROUTINE compev(valuearr, gdataobji, parain, iloop, kloop)  ! given type, time and data (cumafdc, hit, etc.), locate the ev entry
        USE datastore, only: gdataobj, paraobj
        IMPLICIT NONE
        TYPE(gdataobj), INTENT(IN) :: gdataobji
        TYPE(paraobj),INTENT(IN)  ::  parain
        REAL(KIND=8),INTENT(OUT) ::  valuearr(parain%mdim)
        INTEGER,INTENT(IN)    ::  iloop, kloop
        INTEGER    ::  klooptmp, count1, count2
        IF (gdataobji%tlcompute==0 .OR. (gdataobji%tlcompute==1 .AND. kloop .LT. gdataobji%tlstart)) THEN
            CALL setevtp1 (parain%dpsdep, iloop, kloop, gdataobji, parain%discount, 0, (/0,0,0,0,0,0,0,0/) , valuearr)  
        ELSE 
            klooptmp=kloop-gdataobji%tlstart+1            
            IF (gdataobji%tlnlife==0 .OR. (gdataobji%tlnlife==1 .AND. gdataobji%datain(kloop)%cumafdc<gdataobji%tllen) ) THEN
                count1= gdataobji%datain(kloop)%cumafdc
                count2= MIN(gdataobji%datain(kloop)%cumafdc+1,gdataobji%tllen)
                CALL setevtp1 (parain%dpsdep, iloop, klooptmp, gdataobji, parain%discount, 1, &
                    (/count1,count1,count1,count2,count2,count1,count1,count1/) , valuearr)  
            ELSEIF (gdataobji%tlnlife==1 .AND. gdataobji%datain(kloop)%cumafdc==gdataobji%tllen .AND. gdataobji%datain(kloop)%cumhitwait<gdataobji%tlnlifebase) THEN
                count1=gdataobji%datain(kloop)%cumhitwait
                CALL setevtp1 (parain%dpsdep, iloop, klooptmp, gdataobji, parain%discount, 2, &
                    (/count1+1,count1+1,count1+1,count1+1,count1+1,count1+1,count1+1,count1+1/) , valuearr)      
            ELSEIF (gdataobji%tlnlife==1 .AND. gdataobji%datain(kloop)%cumafdc==gdataobji%tllen .AND. gdataobji%datain(kloop)%cumhitwait==gdataobji%tlnlifebase) THEN
                CALL setevtp1 (parain%dpsdep, iloop, klooptmp, gdataobji, parain%discount, 1, (/1,1,1,1,1,1,1,1/) , valuearr)        
            ELSE
                print *, 'ERROR IN SUBROUTINE COMPEV'
                STOP
            END IF
        END IF
        valuearr = valuearr - valuearr(1)     
    END SUBROUTINE compev         

     SUBROUTINE simulatetype(gdataobji, parain, ind, simsize, writenum, simin)
        use mkl95_blas          ! function DOT
        !use mkl95_lapack
        use datastore, only: paraobj, gdataobj, lcobj, simobj
        IMPLICIT NONE                
        TYPE(gdataobj),INTENT(INOUT)  ::  gdataobji
        TYPE(simobj),INTENT(INOUT)     ::  simin        
        TYPE(paraobj),INTENT(IN)  ::  parain
        INTEGER,INTENT(IN)    ::  ind, simsize, writenum  ! ind: program availability indicator (default=4)
        TYPE(lcobj) :: lc        
        INTEGER    ::  factorvalout, iloop, jloop, kloop, mloop, choice
        REAL(KIND=8)    ::  jobprob, wage, randwage, utility, ben(8,4)        
        REAL(KIND=8), DIMENSION(parain%mdim)   ::  income, kernel, valuearr, evvalue   
        REAL(KIND=8)    ::  partiarr(4), hourwage, workscale
        REAL(KIND=8)    ::  wlin, hypwage, hypwage0, hypwage1, hypwage2, hypwage3
        INTEGER        ::  jidx, year 
        do mloop=1,simsize
        factorvalout=catrandnum(parain%sstream, parain%typeprob)     ! randomly generate type : 1, 2, 3, etc.                        
        do kloop=1, gdataobji%gsize        
            CALL complincom(gdataobji%datain(kloop), parain, lc)
            CALL complincomf(lc, parain, factorvalout)
            IF (parain%jelast==0) then
                jobprob = blogit( lc%jlincomf)            
            ELSE
                jobprob = MIN(blogit( lc%jlincomf) *1.1D0,1.D0)
            END IF
            IF (parain%jelasttmp==1 .AND. kloop==1) jobprob = MIN(blogit( lc%jlincomf) *1.1D0,1.D0)
            CALL compev(evvalue, gdataobji, parain, factorvalout, kloop)   ! finds ev in period t+1
            randwage=randnum(parain%sstream, 'N')                           ! randomly generate wage offer
            hypwage=MAX( EXP( randnum(parain%sstream3, 'N') * lc%wstdlincomf + lc%wlincomf), gdataobji%datain(kloop)%minwage)! gen hypothe wage all (include no job offer)
            IF (randnum(parain%sstream, 'U') < jobprob) THEN               ! randomly generate job offer
                IF (parain%welast==0) then
                    wage=MAX( EXP( randwage * lc%wstdlincomf + lc%wlincomf), gdataobji%datain(kloop)%minwage)
                ELSE
                    wage=MAX( EXP( randwage * lc%wstdlincomf + lc%wlincomf), gdataobji%datain(kloop)%minwage) *1.1D0
                END IF
                IF (parain%welasttmp==1) wage=MAX( EXP( randwage * lc%wstdlincomf + lc%wlincomf), gdataobji%datain(kloop)%minwage) *1.1D0
                CALL genincsim(income,wage,gdataobji%datain(kloop),ben, ind)
                valuearr=(lc%lincomf + income + parain%quadratic*income**2.d0 )/(parain%errvar)  + evvalue  ! quad
                CALL logit(kernel,valuearr)                                              
            ELSE
                wage=0.d0                                                   ! no job offer: output wage=0
                CALL genincsim(income,wage,gdataobji%datain(kloop),ben, ind)
                valuearr=(lc%lincomf + income + parain%quadratic*income**2.d0 )/(parain%errvar) + evvalue   ! quad        
                CALL logit(kernel,valuearr)                   
            END IF
            choice = catrandnum(parain%sstream2, kernel)                   
            IF (kloop<gdataobji%gsize) then
                IF (parain%dpsdep==1) THEN
                    gdataobji%datain(kloop+1)%varwork(1)    = indic( IN( choice,(/2,3,5,7,8/) ) ) 
                    gdataobji%datain(kloop+1)%varafdc(1)    = indic( IN( choice,(/4,5/) ) )
                    gdataobji%datain(kloop+1)%varfs(1)      = indic( IN( choice,(/4,5,6,7,8/) ) )                 
                    gdataobji%datain(kloop+1)%varwage(1)    = (indic( IN( choice,(/2,3,5,7,8/) ) ) )*0.1d0  ! COEF NORMALIZED BY 0.1
                    gdataobji%datain(kloop+1)%varjob(1)     = (indic( IN( choice,(/2,3,5,7,8/) ) ) )*0.1d0   ! COEF NORMALIZED BY 0.1
                END IF                   
                IF (gdataobji%tlcompute==1 .AND. kloop .GE. gdataobji%tlstart) THEN
                IF (gdataobji%tlnlife==0 .OR. (gdataobji%tlnlife==1 .AND. gdataobji%datain(kloop)%cumafdc<gdataobji%tllen) ) THEN
                    gdataobji%datain(kloop+1)%cumafdc = MIN(gdataobji%datain(kloop)%cumafdc +  INT(indic(choice==4 .or. choice==5)) ,gdataobji%tllen)
                    gdataobji%datain(kloop+1)%hit = INT( indic(gdataobji%datain(kloop+1)%cumafdc==gdataobji%tllen) )
                    gdataobji%datain(kloop+1)%cumhitwait = 1
                ELSEIF (gdataobji%tlnlife==1 .AND. gdataobji%datain(kloop)%cumafdc==gdataobji%tllen .AND. gdataobji%datain(kloop)%cumhitwait<gdataobji%tlnlifebase) THEN                
                    gdataobji%datain(kloop+1)%cumafdc = gdataobji%tllen
                    gdataobji%datain(kloop+1)%hit = 1
                    gdataobji%datain(kloop+1)%cumhitwait = gdataobji%datain(kloop)%cumhitwait + 1
                ELSEIF (gdataobji%tlnlife==1 .AND. gdataobji%datain(kloop)%cumafdc==gdataobji%tllen .AND. gdataobji%datain(kloop)%cumhitwait==gdataobji%tlnlifebase) THEN
                    gdataobji%datain(kloop+1)%cumafdc = 1
                    gdataobji%datain(kloop+1)%hit = 0
                    gdataobji%datain(kloop+1)%cumhitwait = 1
                ELSE
                    print *, 'ERROR IN SUBROUTINE COMPEV'
                    STOP
                END IF
                END IF
                IF (parain%lcftype==3) THEN    
                    gdataobji%datain(kloop+1)%cumafdc = MIN(gdataobji%datain(kloop)%cumafdc +  INT(indic(choice==4 .or. choice==5)) ,gdataobji%tllen)
                    gdataobji%datain(kloop+1)%hit = INT( indic(gdataobji%datain(kloop+1)%cumafdc==gdataobji%tllen) )
                    gdataobji%datain(kloop+1)%cumhitwait = 1
                END IF                    
            END IF
            utility =  ( MAXVAL(valuearr) + LOG( SUM( EXP( valuearr-MAXVAL(valuearr) ) ) ) - DOT(kernel,evvalue) )*parain%errvar ! avoid overflow
            
            if (parain%dpsdep==1) then
                wlin = lc%wlincomf - gdataobji%datain(kloop)%varwage(1) * parain%pvarwage(1)
            else
                wlin = lc%wlincomf
            end if
            if (parain%writesim==1) then
                write(writenum, '(i5, 1x, 4(i2, 1x), 6(f8.2,1x), 3(i2, 1x), i2, 1x, 10(f8.2,1x) )') gdataobji%id, mloop, kloop+1, factorvalout, choice, &
                    wage, utility, ben(choice,:), &
                    gdataobji%datain(kloop)%cumafdc, gdataobji%datain(kloop)%hit, gdataobji%datain(kloop)%cumhitwait, &
                    gdataobji%gsize, wlin, hypwage !, hypwage0, hypwage1, hypwage2, hypwage3
            end if
            
            ! Compute summary statistics from simulation
            IF (parain%lcftype==0) then    ! IF NOT LONG-RUN SIMULATION, COMPUTE SUMMARY STATISTICS 
                partiarr= (/ INDIC(IN(choice,(/ 2,3,5,7,8 /) )), INDIC(IN(choice, (/3,8/) )), INDIC(IN(choice, (/4,5/) )), INDIC(IN(choice, (/4,5,6,7,8/) )) /)
                hourwage=wage/80.d0           
                workscale = INDIC(IN(choice, (/2,5,7/))) * 1.d0 + INDIC(IN(choice, (/3,8/))) * 2.d0            
                year=INT(gdataobji%datain(kloop)%var(1))    ! calendar year (between 1992 and 1999)  
                simin%chraw(choice)             =simin%chraw(choice)+1.d0
                simin%chyrraw(year-1991,choice) =simin%chyrraw(year-1991,choice)+1
                simin%chwvraw(kloop,choice)     =simin%chwvraw(kloop,choice)+1            
                simin%outyr(year-1991,:)        =simin%outyr(year-1991,:) + partiarr
                simin%outwv(kloop,:)            =simin%outwv(kloop,:) + partiarr          
                simin%wpref=simin%wpref + lc%lincomf(2)
                simin%apref=simin%apref + lc%lincomf(4)
                simin%fpref=simin%fpref + lc%lincomf(6)
                simin%jmean=simin%jmean + lc%jlincomf
                simin%jobprob = simin%jobprob + jobprob
                IF ( IN(choice, (/1,4,6/)) ) simin%jobprob_0 = simin%jobprob_0 + jobprob
                IF ( IN(choice, (/2,3,5,7,8/)) ) simin%jobprob_1 = simin%jobprob_1 + jobprob     
                IF (gdataobji%id<10000) THEN
                    simin%outwv_p96(kloop,:)=simin%outwv_p96(kloop,:)+partiarr
                    simin%cntwv_p96(kloop) = simin%cntwv_p96(kloop)+1
                ELSE
                    simin%outwv_p923(kloop,:)=simin%outwv_p923(kloop,:)+partiarr      
                    simin%cntwv_p923(kloop) = simin%cntwv_p923(kloop)+1
                END IF                           
                simin%outcome                   =simin%outcome+ (/ partiarr, wage*workscale, utility, ben(choice,:) /)
                IF ( IN(choice, (/2,3,5,7,8/)) ) THEN
                    CALL locate2(simin%wageidx,hourwage,jidx)  ! jidx returns 0 to SIZE(simin%wageidx)
                    simin%wagedist(jidx+1) = simin%wagedist(jidx+1) + 1.d0
                    CALL locate2(simin%wageidx_csq,LOG(hourwage),jidx)
                    simin%wagedist_csq(jidx+1) = simin%wagedist_csq(jidx+1) + 1.d0                    
                END IF               
                IF ( IN(choice, (/2,3,5,7,8/)) ) simin%numsimwkr=simin%numsimwkr+1
                IF ( IN(choice, (/1,4,6/)) ) simin%numsimnonwkr=simin%numsimnonwkr+1
                simin%numsim=simin%numsim+1                
            END IF
        end do 
        end do
    END SUBROUTINE simulatetype       
    
    SUBROUTINE siminprocess(path,simin)
        USE datastore, only: simobj       
        IMPLICIT NONE
        TYPE(simobj),INTENT(INOUT) :: simin
        CHARACTER(*),INTENT(IN)   ::  path
        INTEGER    ::  iloop , jloop, ionum=1, iostat
        NAMELIST / instruction/ simin
        open(1, file=trim(path)//'actdist.txt', status='old',iostat=iostat) ! READ ACTUAL DISTRIBUTION INFORMATION FROM EXTERNAL FILE
        print *, 'iostat for opening actdist.txt = ', iostat    
        READ(1,NML=instruction)
        close(1)
        open(ionum, file=trim(path)//'sumstat.txt', status='replace')
        print *, 'iostat for opening sumstat.txt = ', iostat    
        simin%achyrdist = RESHAPE(simin%achoice_csq,(/8,8/))        
        DO iloop=1,12
            IF (iloop .LE. 8) THEN
                IF (SUM(simin%chyrraw(iloop,:))>0) THEN
                    simin%chyrdist(iloop,:) = 100.d0 * simin%chyrraw(iloop,:)/SUM(simin%chyrraw(iloop,:)) ! in percent
                    simin%achyrdist(iloop,:) = 100.d0 * simin%achyrdist(iloop,:)/SUM(simin%achyrdist(iloop,:))
                    simin%outyr(iloop,:) = 100.d0 * simin%outyr (iloop,:)/SUM(simin%chyrraw(iloop,:))     ! in percent
                END IF
            END IF
            IF (SUM(simin%chwvraw(iloop,:))>0) THEN
                simin%chwvdist(iloop,:) = 100.d0 * simin%chwvraw(iloop,:)/SUM(simin%chwvraw(iloop,:))     ! in percent
                simin%outwv(iloop,:) = 100.d0 * simin%outwv (iloop,:)/SUM(simin%chwvraw(iloop,:))         ! in percent
                simin%outwv_p96(iloop,:) = 100.d0 * simin%outwv_p96 (iloop,:)/ DBLE(MAX(simin%cntwv_p96(iloop),1))         ! in percent
                simin%outwv_p923(iloop,:) = 100.d0 * simin%outwv_p923 (iloop,:)/ DBLE(MAX(simin%cntwv_p923(iloop),1))        ! in percent
            END IF
        END DO
        simin%chraw=simin%chraw/DBLE(simin%numsim) * 100.d0
        simin%outcome = simin%outcome/simin%numsim
        simin%outcome(1:4) = 100.d0 * simin%outcome(1:4)                  ! express part rate in percent
        simin%jobprob = simin%jobprob/simin%numsim
        simin%jobprob_0 = simin%jobprob_0/DBLE(simin%numsimnonwkr)
        simin%jobprob_1 = simin%jobprob_1/DBLE(simin%numsimwkr)
        simin%wpref = simin%wpref / DBLE(simin%numsim)
        simin%apref = simin%apref / DBLE(simin%numsim)
        simin%fpref = simin%fpref / DBLE(simin%numsim)
        simin%jmean = simin%jmean / DBLE(simin%numsim)
        simin%wagedist = 100.d0 * simin%wagedist/DBLE(simin%numsimwkr)           ! express wage dist in percent
        simin%wagedist_csq = 100.d0 * simin%wagedist_csq/DBLE(simin%numsimwkr)   ! express wage dist in percent
        simin%choice_csq=RESHAPE(simin%chyrraw,(/64/))
        simin%choice_csq=100.d0 * simin%choice_csq/DBLE(simin%numsim)          ! express choice dist in percent
        CALL csqtest(simin%awagedist_csq,simin%wagedist_csq, simin%wkrsample,simin%wagecsq(1),simin%wagecsq(2),simin%wagecsq(3)) ! total number of workers in sample
        CALL csqtest(simin%achoice_csq,simin%choice_csq, simin%chsample, simin%chcsq(1),simin%chcsq(2),simin%chcsq(3))       ! everyone in sample
        write(ionum,*) "Actual choice distribution by year:"
        write(ionum,'(8(f6.2, 1x))') ( (simin%achyrdist(iloop,jloop),jloop=1,8), iloop=1,8) 
        write(ionum,*) "Simulated choice distribution:"
        write(ionum,'(8(f6.2, 1x))') (simin%chraw(iloop), iloop=1,8) 
        write(ionum,*) "Simulated choice distribution by year:"
        write(ionum,'(8(f6.2, 1x))') ( (simin%chyrdist(iloop,jloop),jloop=1,8), iloop=1,8) 
        write(ionum,*) "Simulated choice distribution by wave:"
        write(ionum,'(8(f6.2, 1x))') ( (simin%chwvdist(iloop,jloop),jloop=1,8), iloop=1,12) 
        write(ionum,*) "Participation rates by year:"
        write(ionum,'(4(f6.2, 1x))') ( (simin%outyr(iloop,jloop),jloop=1,4), iloop=1,8) 
        write(ionum,*) "Participation rates by wave:"
        write(ionum,'(4(f6.2, 1x))') ( (simin%outwv(iloop,jloop),jloop=1,4), iloop=1,12) 
        write(ionum,*) "Outcomes (parti rates, earnings, utility, 4 benefits:"
        write(ionum,'(f8.2, 1x)') (simin%outcome(iloop),iloop=1,10)
        write(ionum,*) "Mean work, welfare and fs pref, jlogitmean; jobprob (all,nonwk,wk):"        
        write(ionum,'(7(f13.7, 1x))') simin%wpref, simin%apref, simin%fpref, simin%jmean, simin%jobprob, simin%jobprob_0, simin%jobprob_1
        write(ionum,*) "Panel 9293 Participation rates by wave:"
        write(ionum,'(4(f6.2, 1x))') ( (simin%outwv_p923(iloop,jloop),jloop=1,4), iloop=1,12)         
        write(ionum,*) "Panel 96 Participation rates by wave:"
        write(ionum,'(4(f6.2, 1x))') ( (simin%outwv_p96(iloop,jloop),jloop=1,4), iloop=1,12)         
        write(ionum,*) "Wage chi-square test statistics, pvalue (pct), degrees of freedom"
        write(ionum,'(3(f8.2, 1x))') (simin%wagecsq(iloop), iloop=1,3)        
        write(ionum,*) "Choice chi-square test statistics, pvalue (pct), degrees of freedom"
        write(ionum,'(3(f8.2, 1x))') (simin%chcsq(iloop), iloop=1,3)
        write(ionum,*) "Total number of observations, number of workers in sample"
        write(ionum,'(2(I5, 1x))') simin%chsample, simin%wkrsample        
        write(ionum,*) "Index, actual and predicted wage distribution (for histograms):"
        write(ionum,'(3(f8.2, 1x))') 4.d0, simin%awagedist(1), simin%wagedist(1)
        write(ionum,'(3(f8.2, 1x))') ( (/simin%wageidx(iloop-1), simin%awagedist(iloop), simin%wagedist(iloop) /), iloop=2,SIZE(simin%awagedist))
        write(ionum,*) "Index, actual and predicted log wage distribution (for chisq test):"
        write(ionum,'(3(f8.2, 1x))') 1.375d0, simin%awagedist_csq(1), simin%wagedist_csq(1)
        write(ionum,'(3(f8.2, 1x))') ( (/simin%wageidx_csq(iloop-1), simin%awagedist_csq(iloop), simin%wagedist_csq(iloop) /), iloop=2,SIZE(simin%awagedist_csq))

        close(ionum)
    END SUBROUTINE siminprocess
        
    SUBROUTINE csqtest(actarr,predarr,N,csqstat,pvalue,df)   ! perform chi-square test on equally sized samples
        USE CHIDF_INT
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  actarr(:),predarr(:)    ! arrays elements as PERCENTAGE of sample
        INTEGER,INTENT(IN)    ::  N   ! size of sample
        REAL(KIND=8),INTENT(OUT)   ::  csqstat, pvalue,df
        INTEGER    ::  iloop
        REAL(KIND=8)    ::  temp
        IF (SIZE(actarr) .NE. SIZE(predarr)) THEN
            PRINT *,'ERROR IN CHISQTEST'
            STOP
        END IF
        df=0.d0
        csqstat=0.d0
        DO iloop=1,SIZE(actarr)
            temp=(actarr(iloop)-predarr(iloop))**2.d0/predarr(iloop)
            IF (ABS(actarr(iloop))>1.d-3) THEN           ! skip empty cells (if present)
                csqstat=csqstat + temp
                df=df+1.d0
            END IF
        END DO
        df=df-1.d0
        csqstat = csqstat * N / 100.d0
        pvalue = 100.d0* (1.D0- CHIDF(csqstat,df))
    END SUBROUTINE csqtest

    SUBROUTINE checkparamvalue(parain)
        USE datastore, only: paraobj
        IMPLICIT NONE
        TYPE(paraobj),INTENT(IN)  ::  parain
        IF (IN(parain%simstatus,(/0,1,2,3/)) == .FALSE.)    GOTO 901
        IF (IN(parain%cdynprog,(/0,1/)) == .FALSE.)         GOTO 902
        IF (IN(parain%cjob,(/0,1/)) == .FALSE.)             GOTO 903
        IF (IN(parain%csanction,(/0,1/)) == .FALSE.)        GOTO 904
        IF (IN(parain%ctype,(/0,1/)) == .FALSE.)            GOTO 905
        IF (IN(parain%cdisc,(/0,1/)) == .FALSE.)            GOTO 906
        IF (IN(parain%cquad,(/0,1/)) == .FALSE.)            GOTO 907
        IF (IN(parain%dpsdep,(/0,1/)) == .FALSE.)           GOTO 908
        IF (IN(parain%cbig,(/0,1/)) == .FALSE.)           GOTO 909
        return
        901 print *, 'simstatus not initialized. program stop.'
        goto 999
        902 print *, 'cdynprog not initialized. program stop.'
        goto 999
        903 print *, 'cjob not initialized. program stop.'
        goto 999
        904 print *, 'csanction not initialized. program stop.'
        goto 999
        905 print *, 'ctype not initialized. program stop.'
        goto 999
        906 print *, 'cdisc not initialized. program stop.'
        goto 999
        907 print *, 'cquad not initialized. program stop.'
        goto 999
        908 print *, 'dpsdep not initialized. program stop.'
        goto 999
        909 print *, 'cbig not initialized. program stop.'
        999 stop        
    END SUBROUTINE checkparamvalue     
    
    SUBROUTINE LIKEVFCN(M,N,X,F)
        !use subroutines, ONLY: likefcn
        USE DATASTORE, ONLY:   farr
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  M,N
        REAL(KIND=8),INTENT(IN)  ::  X(N)
        REAL(KIND=8),INTENT(OUT) ::  F(M)        
        REAL(KIND=8)  ::  fvalue
        CALL LIKEFCN(N,X,fvalue)
        F=-farr
    END SUBROUTINE LIKEVFCN    

    FUNCTION inverse(A,N)
        USE LINRG_INT
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  N   ! Gives the dimension of the input matrix
        REAL(KIND=8),DIMENSION(N,N)    ::  inverse            
        REAL(KIND=8),DIMENSION(N,N),INTENT(IN)  ::  A
        CALL LINRG(A,inverse)
    END FUNCTION inverse
            
    SUBROUTINE std(paravec,parastd,covmat,datasize,paranum)
        USE FDJAC_INT
        USE MXTXF_INT
        USE datastore, only: path
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  datasize,paranum                
        REAL(KIND=8),INTENT(IN)  ::  paravec(paranum)
        REAL(KIND=8),INTENT(OUT) ::  parastd(paranum)     
        REAL(KIND=8),INTENT(OUT)   ::  covmat(paranum,paranum)
        REAL(KIND=8)  ::  vfvalue(datasize), fjac(datasize,paranum), xscale(paranum)
        REAL(KIND=8)    ::  temp(paranum,paranum)
        INTEGER    ::  iloop,jloop
        CALL likevfcn(datasize,paranum,paravec,vfvalue)
        xscale = ABS(1.D0/paravec)   ! define scale for std err calculation            
        CALL FDJAC(likevfcn, paravec, vfvalue, fjac, XSCALE=xscale, EPSFCN=0.00D0)
        open(30, file=trim(path)//'fjac.txt',status='replace')
        DO iloop=1,datasize
            WRITE(30,'(100f15.7)') (fjac(iloop,jloop), jloop=1,paranum)
        END DO         
        close(30)
        CALL MXTXF (fjac, temp)
        covmat = inverse(temp, paranum)
        !stdmat = inverse( matmul(transpose(fjac), fjac), paranum )
        DO iloop=1,paranum
            parastd(iloop) = sqrt(covmat(iloop, iloop))
        END DO
    END SUBROUTINE std    
        
    PURE SUBROUTINE locate(xx,n,x,j) ! n is the size of xx minus 1
        implicit none
        integer,intent(in)    ::  n
        integer,intent(out)   ::  j
        real(kind=8),intent(in)   ::  x,xx(0:n)
        DO j=0,n
            IF (x .LT. xx(j)) EXIT
        END DO
    END SUBROUTINE locate     
    PURE SUBROUTINE locate2(xx,x,j) ! xx is array, x is input, j takes value from 0 to SIZE(xx)
        implicit none
        integer,intent(out)   ::  j
        real(kind=8),intent(in)   ::  x,xx(:)
        DO j=0,SIZE(xx)-1
            IF (x .LT. xx(j+1)) EXIT
        END DO
    END SUBROUTINE locate2     
    
    PURE FUNCTION interp(arrsize,xx,yy,x)
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  arrsize        
        REAL(KIND=8),INTENT(IN),DIMENSION(arrsize)  ::  xx,yy
        REAL(KIND=8),INTENT(IN)  ::  x
        REAL(KIND=8)  ::  interp
        INTEGER    ::  index
        CALL locate(xx,arrsize-1,x,index)
        IF ((index .GT. 0) .AND. (index .LT. arrsize)) THEN
            interp = yy(index) + (yy(index+1)-yy(index))/(xx(index+1)-xx(index))*(x-xx(index))                                  
        ELSE IF (index .EQ. 0) THEN
            interp = yy(1) + (yy(2)-yy(1))/(xx(2)-xx(1))*(x-xx(1))
        ELSE
            interp = yy(arrsize-1) + (yy(arrsize)-yy(arrsize-1))/(xx(arrsize)-xx(arrsize-1))*(x-xx(arrsize-1))
        END IF
    END FUNCTION interp        

    PURE FUNCTION indic(logic)
        IMPLICIT NONE
        LOGICAL,INTENT(IN)    ::  logic
        REAL(KIND=8)    ::  indic
        IF (logic .EQ. .TRUE.) THEN
            indic = 1.D0
        ELSE
            indic = 0.D0
        END IF
    END FUNCTION indic                        
    
    REAL(KIND=8) FUNCTION ncdf(x)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  x
        REAL(KIND=8)    ::  t, f
        REAL(KIND=8), PARAMETER    ::  p=0.2316419d0, b1=0.31938d0, b2=-0.35656d0, b3=1.78148d0, b4=-1.82125d0, b5=1.33027d0, nconstant=0.39894228d0
        t=1.d0/(1.d0+p*x)
        f=nconstant*EXP(-(x**2/2.d0))
        ncdf = 1.d0 - f*( b1*t + b2*t**2 + b3*t**3 + b4*t**4 + b5*t**5)
    END FUNCTION ncdf        
    
    REAL(KIND=8) FUNCTION blogit(x)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  x
        IF (x>0.d0) THEN           ! AVOID OVERFLOW
            blogit = 1.d0 / ( 1.d0 + EXP(-x) )
        ELSE
            blogit = exp(x) / ( 1.d0 + EXP(x) )
        END IF
    END FUNCTION blogit
         
    SUBROUTINE logit(out,arr,logicarr)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  arr(:)
        LOGICAL,optional, INTENT(IN)    ::  logicarr(:)
        REAL(KIND=8),INTENT(OUT)   ::  out(:)
        REAL(KIND=8)    ::  base, nomin(size(arr)), arrnorm(size(arr))
        arrnorm=arr-MAXVAL(arr)          ! normalize to avoid overflow
        nomin=EXP(arrnorm)              ! array of numerators
        base=SUM(nomin)                 ! sum of array of numerators
        out=nomin/base
    END SUBROUTINE logit       
             
     REAL(KIND=8) FUNCTION randnum(stream, rtype)
        use mkl_vsl
        use mkl_vsl_type
        IMPLICIT NONE
        TYPE (VSL_STREAM_STATE), INTENT(IN) :: stream
        CHARACTER(1),INTENT(IN)   ::  rtype
        REAL(KIND=8)    ::  temp(1)
        INTEGER    ::  iostat
        IF (rtype == 'N') THEN
            iostat=vdrnggaussian( VSL_METHOD_DGAUSSIAN_BOXMULLER2, stream, 1, temp, 0.d0, 1.d0 )
        ELSEIF (rtype == 'U') THEN
            iostat=vdrnguniform( VSL_METHOD_DUNIFORM_STD_ACCURATE, stream, 1, temp, 0.d0, 1.d0 )
        ELSE 
            PRINT *, 'error randnum type "U or N"!'
        END IF
        if (iostat /= vsl_error_ok) print *, 'error produce stream'             
        randnum=temp(1)       
    END FUNCTION randnum
    
    INTEGER FUNCTION catrandnum(stream, probin)
        use mkl_vsl_type
        IMPLICIT NONE
        TYPE (VSL_STREAM_STATE),INTENT(IN) :: stream
        REAL(KIND=8), INTENT(IN)   ::  probin(:)
        INTEGER    ::  arrsize, iloop, done
        REAL(KIND=8)    ::  uniform, probarr(SIZE(probin))
        arrsize=size(probin)
        if (ABS(SUM(probin)-1.d0) >1.d-4 ) print *, 'error catrandnum'
        probarr(1)=probin(1)
        !if (arrsize>1) then
            do iloop=2,arrsize
                probarr(iloop)=probarr(iloop-1)+probin(iloop)
            end do        
        !end if
        uniform=randnum(stream, 'U')
        done=0
        do iloop=1,arrsize
            if (  ( uniform .LT. probarr(iloop) )    .AND. done==0) THEN
                catrandnum=iloop
                done=1
            end if
        end do        
        if (done==0) catrandnum=arrsize
    END FUNCTION catrandnum    
    
    LOGICAL FUNCTION IN(intin, intlist)
        IMPLICIT NONE
        INTEGER,INTENT(IN)    ::  intin, intlist(:)
        INTEGER    ::  iloop       
        in=.false.
        do iloop=1,size(intlist)
            if (intin .EQ. intlist(iloop)) then
                in=.true.
                exit
            end if
        end do
    END FUNCTION IN
    
    REAL(KIND=8) FUNCTION pvalue(zscore,side)      ! computes pvalue (in percent)
        use anordf_int          ! computes normal cdf
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  zscore  ! supply positive or negative t-stat
        INTEGER,INTENT(IN)    ::  side        ! side=1 (one-sided test), other values (two-sided test, default)
        IF (side==1) THEN
            pvalue = 1.D0 - anordf( ABS(zscore) )*100.d0
        ELSE
            pvalue = (1.D0 - anordf( ABS(zscore) ))*2.D0*100.d0       
        END IF
    END FUNCTION pvalue  
    CHARACTER(3) FUNCTION pvaluestar(pvalue)
        IMPLICIT NONE
        REAL(KIND=8),INTENT(IN)    ::  pvalue      ! pvalue in percent
        IF (pvalue<=1.d0) THEN
            pvaluestar='***'
        ELSEIF (pvalue >1.d0 .AND. pvalue <= 5.d0) THEN
            pvaluestar='** '
        ELSEIF (pvalue >5.d0 .AND. pvalue <= 10.d0) THEN
            pvaluestar='*  '
        ELSE
            pvaluestar='   '
        END IF
    END FUNCTION pvaluestar

    ! Obselete subroutine 2012
    SUBROUTINE initlongsim(gdataobji)
        use datastore, only: gdataobj, maxgsize    
        IMPLICIT NONE
        TYPE(gdataobj),INTENT(INOUT)  ::  gdataobji       ! Update dataset (hence the INOUT)
        INTEGER    ::  kloop
        REAL(KIND=8)    ::  temp1, temp2, temp3, temp4, temp5
        gdataobji%tlhor=11*3        ! Simulate individuals for 11 years
        gdataobji%tlcompute=0       ! time limit existence indicator   
        gdataobji%tllen=15          ! length of time limit             
        gdataobji%tlnlife=0         ! non-lifetime limit indicator
        gdataobji%tlred=0           ! benefit-reduction limit indicator
        gdataobji%tlstart=1         ! start time limit in wave 1        
        gdataobji%horizon=gdataobji%tlhor       
        gdataobji%gsize=maxgsize
        
        if (gdataobji%id .GE. 10000) THEN              
            gdataobji%datain(8)%varwork(8)=0.d0         
            gdataobji%datain(8)%varwork(9)=1.d0
            gdataobji%datain(8)%varafdc(9)=0.d0
            gdataobji%datain(8)%varafdc(10)=1.d0
            gdataobji%datain(8)%varfs(10)=0.d0
            gdataobji%datain(8)%varfs(11)=1.d0
            gdataobji%datain(8)%varwage(9)=0.d0
            gdataobji%datain(8)%varwage(10)=1.d0
            gdataobji%datain(8)%varjob(10)=0.d0
            gdataobji%datain(8)%varjob(11)=1.d0
            gdataobji%datain(8)%rfoklt18=gdataobji%datain(1)%rfoklt18
            gdataobji%datain(1)=gdataobji%datain(8)      
        END IF
        gdataobji%datain(1)%tlred=0
        
        gdataobji%datain(1)%cumafdc=1
        gdataobji%datain(1)%hit=0
        do kloop=1,maxgsize
            temp1 = gdataobji%datain(kloop)%varwork(1)
            temp2 = gdataobji%datain(kloop)%varafdc(1)
            temp3 = gdataobji%datain(kloop)%varfs(1)
            temp4 = gdataobji%datain(kloop)%varwage(1)
            temp5 = gdataobji%datain(kloop)%varjob(1)
            
            gdataobji%datain(kloop)=gdataobji%datain(1)         
            
            gdataobji%datain(kloop)%varwork(1)=temp1
            gdataobji%datain(kloop)%varafdc(1)=temp2
            gdataobji%datain(kloop)%varfs(1)=temp3
            gdataobji%datain(kloop)%varwage(1)=temp4
            gdataobji%datain(kloop)%varjob(1)=temp5
 
        end do
    END SUBROUTINE initlongsim            
        
END MODULE SUBROUTINES