! 2012/01/13 Marc Chan
! This program reads income tax parameters in namelist format and prints parameters in rectangular format (by state and year)
! Output file variable layout: state name, state code, year, state income tax indicator, standard deduction, personal exemption, dependent exemption,
!   tax bracket (1 to 6), tax rate (1 to 6), max tax in bracket (1 to 6), 
!   state eitc indicator, state eitc refundable indicator, state eitc percent proportion (1 to 3)

!stax%sname, stax%scode, jloop, stax%tax, stax%ded, stax%exp, stax%dexp, stax%bracket(jloop,:), stax%rate(jloop,:), stax%maxben(jloop,:), &
!                    stax%seitc(jloop,1), stax%seitc(jloop,2), stax%seitcben(jloop,:)

program sample
    implicit none
    INTEGER    ::  iostat=0, iloop, jloop    
    ! Define object containing sate income tax parameters
    TYPE staxobj                                   ! state income tax parameter
        CHARACTER(20)  ::  sname=''                ! name of state
        INTEGER        ::  scode=0                 ! code (SIPP96) of state
        REAL(KIND=8)    ::  ded=0.d0                ! standard deduction (as indicated in year 2000, tax foundation)
        REAL(KIND=8)    ::  exp=0.d0                ! personal exemption (as indicated in year 2000, tax foundation)
        REAL(KIND=8)    ::  dexp=0.d0               ! dependent exemption (as indicated in year 2000, tax foundation)
        INTEGER        ::  tax=0                  ! indicator of whether a state income tax exists
        REAL(KIND=8),DIMENSION(1992:1999,12)    ::  raw=0.d0  ! raw tax brackets and rates (MAX 6 brackets - bracket1, rate1, bracket2, rate2, etc.)
        REAL(KIND=8),DIMENSION(1992:1999,6)    ::  bracket=0.d0, rate=0.d0, maxben=0.d0
        INTEGER,DIMENSION(1992:1999,2)   ::  seitc=0             ! column 1: indicator of state eitc. column 2: indicator (1 refundable, 0 non-refundable)
        REAL(KIND=8),DIMENSION(1992:1999,3)    ::  seitcben=0.d0   ! column1: state eitc ben (as proportion of federal eitc) 1 child; ben 2 children, 3+ children        
    END TYPE
    TYPE(staxobj) ::  stax, staxarr(62),staxempty
    namelist / list / stax
    CHARACTER(50)  ::  infile='stax_input.txt', outfile='stax_output.txt'
    
    ! Read income tax parameters
    open (1, file=infile,status='old',iostat=iostat)
    print *, 'OPEN State Income Tax Parameter Input File iostat = ',iostat    
    DO
        stax=staxempty                  ! Initialize state income tax parameters
        read(1,nml=list, iostat=iostat) ! read parameters from external file
        print *, 'Reading ', stax%sname, ' iostat = ', iostat
        IF (stax%scode==-1) EXIT       ! If reach end-of-file directive then exit loop        
        IF (stax%tax==1) THEN          ! process parameters if state income tax exists
        DO iloop=1992,1999
            DO jloop=1,12
            IF ( ABS(stax%raw(iloop,jloop))< 0.01d0 ) THEN
                stax%raw(iloop,jloop)=stax%raw(MAX(iloop-1,1992),jloop)
            END IF
            END DO         
        END DO
        END IF        
        ! Create entries for state income tax brackets, rates and maximum tax payment in each bracket
        DO iloop=1992,1999
            DO jloop=1,6
                stax%bracket(iloop,jloop)=stax%raw(iloop,(jloop-1)*2+1)
                stax%rate(iloop,jloop)=stax%raw(iloop,(jloop-1)*2+2)
                ! Compute max tax payment (xi) in each tax bracket (bi)
                ! x1=b1*r1; x2=x1+(b2-b1)*r2; x3=x2+(b3-b2)*r3, etc.               
                IF (jloop==1) THEN
                    stax%maxben(iloop,1)= stax%bracket(iloop,1)*stax%rate(iloop,1)/100.d0
                ELSE
                    stax%maxben(iloop,jloop)= stax%maxben(iloop,jloop-1) + (stax%bracket(iloop,jloop) - stax%bracket(iloop,jloop-1)) *stax%rate(iloop,jloop)/100.d0
                END IF
            END DO
        END DO
        staxarr(stax%scode)=stax        ! save parameters to the corresponding entry in object array
    END DO   
        
    ! Print income tax parameters
    OPEN (1,file=outfile, status='replace', iostat=iostat)
    print *, 'OPEN State Income Tax Parameter Output File iostat = ',iostat
    DO iloop=1,SIZE(staxarr)
        stax=staxarr(iloop)
        IF ( stax%scode .NE. 0 ) THEN            ! further process results only if there is information            
            DO jloop=1992,1999
                WRITE(1,999) stax%sname, stax%scode, jloop, stax%tax, stax%ded, stax%exp, stax%dexp, stax%bracket(jloop,:), stax%rate(jloop,:), stax%maxben(jloop,:), &
                    stax%seitc(jloop,1), stax%seitc(jloop,2), stax%seitcben(jloop,:)
            END DO
        END IF
    END DO
    999 format (a20, 1x, i2, 1x, i4, 1x, i2, 1x, 3(f8.2,1x), 18(f8.2,1x), 2(i2,1x), 3(f8.2,1x) )
        
    PRINT *, 'Program ended sucessfully.'
end program sample

