/* last edited Marc Chan 2012/08/31: this file reads SAS data and outputs data in txt format */
/* Be sure to add subdirectories in root path for output data before running the code */
options pagesize=40 font='courier new' 8;
libname dataout "c:\tl2010";   /* specify directory of SAS data */
%let outroot=c:\tl2010\fortrancode\basemodel\; 		/* specify root path for output data */

/*********************************************************************************************/
/* this macro creates actdist.txt from panel */
%macro actdist;
data temp;
	set panel;
	if wage>0 then do;
		logwage=min(log(wage/80),2.99);		* define log hourly wage for chi-square analysis;
		wageplot=min(wage/80,20.99); 		* define hourly wage for histogram;
	end;
run;
proc univariate data=temp;	* Plot wage histogram frequencies;
	var wageplot;
   histogram wageplot / nochart midpercents
                     endpoints = 4 to 20.5 by 0.5
					 outhistogram= outhist;
	where wageplot ne .;
run;
proc univariate data=temp;	* wage chi-square test frequencies;
	var logwage;
   histogram logwage / nochart midpercents
                     endpoints = 2 to 3 by 0.125
					 outhistogram= outhist_csq;
	where logwage ne .;
run;
/* start printing instructions for fortran */
data _null_;
	file "&outpath.actdist.txt";
	put "& instruction";
run;
data _null_;  * print actual wage distribution for chi-sq test;
	set outhist_csq end=eof;
	file "&outpath.actdist.txt" mod;
	if _n_=1 then put "simin%awagedist_csq=";
	put (_obspct_ ) (8.4) @;
	if eof=0 then put "," @;
	if _n_=1 then cumfreq=_count_;
	else cumfreq+_count_;
	if eof=1 then put / "simin%wkrsample=" (cumfreq)(8.0);
run;
data _null_;  * print actual wage distribution;
	set outhist end=eof;
	file "&outpath.actdist.txt" mod;
	if _n_=1 then put "simin%awagedist=";
	put (_obspct_ ) (8.4) @;
	if eof=0 then put "," @;
run;
proc freq data=temp noprint; 		* compute choice distribution frequencies;
	tables &choice*yearnew / out=temp1;
run;
data temp2;							* construct data with 64 observations;
	do &choice=1 to 8;
		do yearnew=1992 to 1999;
			output;
		end;
	end;
run;
proc sql;
	create table temp3 as
	select p.*,l.*
	from temp2 as p left join temp1 as l
	on p.yearnew=l.yearnew & p.&choice=l.&choice;
quit;
data temp3;
	set temp3;
	if count=. then count=0;
	if percent=. then percent=0;
run;
proc sort data=temp3;
	by &choice yearnew;
run;
data chyr; 							* print choice distribution;
	set temp3 end=eof;
	file "&outpath.actdist.txt" mod;
	if _n_=1 then put "simin%achoice_csq=";
	put (percent) (8.4) @;
	if eof=0 then put "," @;
	if _n_=1 then cumfreq=count;
	else cumfreq+count;
	if eof=1 then put / "simin%chsample=" (cumfreq)(8.0);
run;
data _null_;
	file "&outpath.actdist.txt" mod;
	put "/";
run;
%mend;

/*********************************************************************************************/
/* Cross-section 1992 model */
%let datatype=static; 							* specify: "dyn" or "static" source data set;
%let outpath=&outroot.cs92\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
	where id>10000;
run;
data panel;
	set dataout.f&datatype;
	where swave=1 & id>10000;
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put "1  " (horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent)(13.7); *6 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent tanfcost)(13.7); *7 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent)(13.7); *6 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (white tage hs south metro avgwage_0100 empshare_8)(13.7); *7 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (white tage hs south metro value valuel3 valuel6 )(13.7); *8 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* Reduced-form Dynamic model with types (no DP at all) */
%let datatype=dyn; 												* dynamic or static data set;
%let outpath=&outroot.rfdynamic\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
run;
data panel;
	set dataout.f&datatype;
	tlredcurrent=tlred*tlcurrent;
	tlm=tlcurrent*max(18-mtagecurrent-tllen/3,0);
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver tlcurrent tlredcurrent tlm)(13.7); *14 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* structural static model with types */
%let datatype=static; 												* dynamic or static data set;
%let outpath=&outroot.strstatic\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
run;
data panel;
	set dataout.f&datatype;
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7);	*9 variables;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7);	*11 variables;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *11 variables;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *10 variables;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*11 variables;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* structural dynamic model with types */
%let datatype=dyn; 												* dynamic or static data set;
%let outpath=&outroot.strdynamic\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
run;
data panel;
	set dataout.f&datatype;
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* structural/reduced form dynamic model with types (using 9293 panels only) */
%let datatype=dyn; 												* dynamic or static data set;
%let outpath=&outroot.strdynamic9293\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
	where id>10000;
run;
data panel;
	set dataout.f&datatype;
	where id>10000;
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 )(13.7); *8 variables ;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 tanfcost)(13.7); *9 variables ;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 )(13.7); *8 variables;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 metro avgwage_0100 empshare_8)(13.7); *9 variables;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 metro value valuel3 valuel6 )(13.7);	*10 variables;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	* no sanction variables;
run;

/*********************************************************************************************/
/* structural/reduced form dynamic model with types (96 panel only) */
%let datatype=dyn; 												* dynamic or static data set;
%let outpath=&outroot.strdynamic96\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
	where id<10000;
run;
data panel;
	set dataout.f&datatype;
	where id<10000;
run;
%let choice=choice;
%actdist;
data _null_;	* group data (1026);
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data (10965);
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9899)(13.7); *8 variables;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9899 tanfcost diver)(13.7); *10 variables;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9899 fsrecert fsebt )(13.7); *10 variables;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9899 metro avgwage_0100 empshare_8)(13.7); *9 variables;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9899 metro value valuel3 valuel6 )(13.7);	*10 variables;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* structural dynamic model with types - change hour definition */
%let datatype=dyn; 												* dynamic or static data set;
%let outpath=&outroot.strdynamic_hour\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
run;
data panel;
	set dataout.f&datatype;
run;
%let choice=newchoice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (newchoice) (2.0);							* new hour definition;
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* structural dynamic model with types - new york state no time limit */
%let datatype=dyn; 												* dynamic or static data set;
%let outpath=&outroot.strdynamic_ny\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
run;
data panel;
	set dataout.f&datatype;
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute_ny tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);	* new york state no time limit;
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* structural dynamic model with types - welfare tax rate reduced by 50 percent */
%let datatype=dyn;												* dynamic or static data set;
%let outpath=&outroot.strdynamic_wtax\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
run;
data panel;
	set dataout.f&datatype;
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);  * change welfare tax rate;
	put (threshold bestd bemax bendol benpct_wtax bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* Cross-section 1992 model (no job offer) */
%let datatype=static; 							* specify: "dyn" or "static" source data set;
%let outpath=&outroot.cs92nj\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
	where id>10000;
run;
data panel;
	set dataout.f&datatype;
	where swave=1 & id>10000;
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put "1  " (horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent value valuel3 valuel6)(13.7); *9 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent tanfcost)(13.7); *7 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent)(13.7); *6 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (white tage hs south metro avgwage_0100 empshare_8)(13.7); *7 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (white tage hs south metro value valuel3 valuel6 )(13.7); *8 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* Cross-section 1992 model (small model) */
%let datatype=static; 							* specify: "dyn" or "static" source data set;
%let outpath=&outroot.cs92small\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
	where id>10000;
run;
data panel;
	set dataout.f&datatype;
	where swave=1 & id>10000;
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put "1  " (horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent value valuel3 valuel6)(13.7); *9 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent tanfcost)(13.7); *7 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent)(13.7); *6 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (white tage hs south metro avgwage_0100 empshare_8)(13.7); *7 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (value valuel3 valuel6 white tage hs south metro  )(13.7); *8 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/*********************************************************************************************/
/* Cross-section 1992 model (unemployment rate affect wage and job offers) */
%let datatype=static; 							* specify: "dyn" or "static" source data set;
%let outpath=&outroot.cs92unemp\;		 			* output file directory;
data panelg;
	set dataout.fg&datatype;
	where id>10000;
run;
data panel;
	set dataout.f&datatype;
	where swave=1 & id>10000;
run;
%let choice=choice;
%actdist;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put "1  " (horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent value valuel3 valuel6)(13.7); *9 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent tanfcost)(13.7); *7 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (white tage hs south rfoklt18 mtagecurrent)(13.7); *6 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (white tage hs south metro avgwage_0100 empshare_8 value valuel3 valuel6)(13.7); *7 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (white tage hs south metro value valuel3 valuel6  )(13.7); *8 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;
