/******************************************************************/
/* modified by marc chan 20120319 (updated 20120405) */
/* Generate counterfactual datasets for counterfactual (1992) analysis */
/* Input: policy.macroecon_state, policy.macroecon_msa, policy.policy */
/*****************************************************************************************************/
libname policy "I:\tl2010\otherpolicydata";
options pagesize=40 font='courier new' 8;
libname dataout "i:\tl2010";
%let outroot=i:\tl2010\fortrancode\basemodel\;

/*****************************************************************************************/
/* Specify input data: structural dynamic model with types */
%let modeltype=strdynamic;
%let datatype=dyn; 												* dynamic or static data set;
data panelg;
	set dataout.fg&datatype;
run;
data panel;
	set dataout.f&datatype;
run;

/* keep demographic variables at second (i.e. first) wave level */
data temp1;
	set panel;
	keep id white tage hs south rfoklt18 mtagecurrent;
	rename white=white_92
	tage=tage_92
	hs=hs_92
	south=south_92
	rfoklt18=rfoklt18_92
	mtagecurrent=mtagecurrent_92;
	where swave=2;
run;
proc sql;
	create table temp2 as
	select p.*, l.*
	from panel as p, temp1 as l
	where p.id=l.id;
quit;
data panel;
	set temp2;
run;

/* no time limit */
data panelg;
	set panelg;
	tlcompute_92=0;
run;

/* no work requirement */
data panel;
	set panel;
	sanction1_92=0;
	sanction2_92=0;
	sanction3_92=0;
run;

/* keep welfare benefit disregards and at 1992 May level (merge with policy.policy) */
proc sql;
	create table temp1 as
	select p.*, l.bendol as bendol_92, l.benpct as benpct_92
	from panel as p left join policy.policy as l
	on p.tfipsst = l.tfipsst &
	l.year=1992 &
	l.month=5 &
	p.rfoklt18+1=l.size;
quit;	
proc sort data=temp1 out=panel;
	by id swave;
run;

/* keep eitc (federal and state) at 1992 May level (merge with policy.policy) */
proc sql;
	create table temp1 as
	select p.*, l.eitcb1 as eitcb1_92, l.eitcb2 as eitcb2_92, l.eitcb3 as eitcb3_92, 
		l.eitcr1 as eitcr1_92, l.eitcr2 as eitcr2_92, l.eitcbm as eitcbm_92, 
		l.seitc as seitc_92, l.seitcref as seitcref_92, l.seitcp as seitcp_92
	from panel as p left join policy.policy as l
	on p.tfipsst = l.tfipsst &
	l.year=1992 &
	l.month=5 &
	p.rfoklt18+1=l.size;
quit;	
proc sort data=temp1 out=panel;
	by id swave;
run;

/* keep minimum wage at 1992 level */
data panel;
	set panel;
	fedminwage_92=4.25;
	fedminwageadj_92=fedminwage_92*80;
run;

/* keep unemployment rate at 1992 May level */
data temp1 temp2;
	set panel;
	if tmsa in (10,41,78,1760,2640,3680,4680,5480,6920,6960,7120,8520,8840,9240) then do;
		tmsaraw=tmsa;
		tmsa=0;
	end;
	if tmsa=0 then output temp1;
	else output temp2;
run;
proc sql;
	create table temp1a as
	select p.*, l.value as value_92
	from temp1 as p left join policy.macroecon_state as l
	on p.tfipsst=l.statecode &
	l.year=1992 &
	l.period=5;
quit;
proc sql;
	create table temp2a as
	select p.*, l.value as value_92
	from temp2 as p left join policy.macroecon_msa as l
	on p.tmsa=l.msa_codenum &
	l.year=1992 &
	l.period=5;
quit;
data panel;
	set temp1a temp2a;
	valuel3_92=value_92;
	valuel6_92=value_92;
run;
proc sort data=panel;
	by id swave;
run;

/* keep wage and industry structure at 1992 May level */
data temp1 temp2;
	set panel;
	if tmsa in (10,41,78,1760,2640,3680,4680,5480,6920,6960,7120,8520,8840,9240) then do;
		tmsaraw=tmsa;
		tmsa=0;
	end;
	if tmsa=0 then output temp1;
	else output temp2;
run;
proc sql;
	create table temp1a as
	select p.*, l.avgwage_0 as avgwage_0_92, l.empshare_8 as empshare_8_92
	from temp1 as p left join policy.macroecon_state as l
	on p.tfipsst=l.statecode &
	l.year=1992 &
	l.period=5;
quit;
proc sql;
	create table temp2a as
	select p.*, l.avgwage_0 as avgwage_0_92, l.empshare_8 as empshare_8_92
	from temp2 as p left join policy.macroecon_msa as l
	on p.tmsa=l.msa_codenum &
	l.year=1992 &
	l.period=5;
quit;
data panel;
	set temp1a temp2a;
	avgwage_0100_92=avgwage_0_92/100;
run;
proc sort data=panel;
	by id swave;
run;

/* keep all other variables at wave 1 or 1992 level */
data temp1;
	set panel;
	keep id metro;
	where swave=2;
run;
proc sql;
	create table temp2 as
	select p.*, l.metro as metro_92
	from panel as p, temp1 as l
	where p.id=l.id;
quit;
data panel;
	set temp2;
run;
proc sql;
	create table temp1 as
	select p.*, 
	l.tanfcost as tanfcost_92, l.diver as diver_92, l.fsrecert as fsrecert_92, l.fsebt as fsebt_92,
	l.threshold as threshold_92, l.bestd as bestd_92, l.bemax as bemax_92, l.bc_bepct as bc_bepct_92,
	l.fsge as fsge_92, l.fsne as fsne_92, l.fsmb as fsmb_92, l.hded as hded_92,
	l.ftaxr1 as ftaxr1_92, l.ded as ded_92,
	l.stax as stax_92, l.nbracket as nbracket_92, l.staxdedttl as staxdedttl_92, 
	l.staxb1 as staxb1_92, l.staxb2 as staxb2_92, l.staxb3 as staxb3_92, 
	l.staxb4 as staxb4_92, l.staxb5 as staxb5_92, l.staxb6 as staxb6_92, 
	l.staxr1 as staxr1_92, l.staxr2 as staxr2_92, l.staxr3 as staxr3_92, 
	l.staxr4 as staxr4_92, l.staxr5 as staxr5_92, l.staxr6 as staxr6_92, 
	l.staxmb1 as staxmb1_92, l.staxmb2 as staxmb2_92, l.staxmb3 as staxmb3_92, 
	l.staxmb4 as staxmb4_92, l.staxmb5 as staxmb5_92, l.staxmb6 as staxmb6_92
	from panel as p left join policy.policy as l
	on p.tfipsst = l.tfipsst &
	l.year=1992 &
	l.month=5 &
	p.rfoklt18+1=l.size;
quit;	
proc sort data=temp1 out=panel;
	by id swave;
run;

data panel;
	set panel;
	notimedummy=0;
run;
/*
data dataout.panel92;
	set panel;
run;
data dataout.panelg92;
	set panelg;
run;*/

/* create directories */
option noxwait;
x "mkdir &outroot.&modeltype._cf92_1";
x "mkdir &outroot.&modeltype._cf92_2";
x "mkdir &outroot.&modeltype._cf92_3";
x "mkdir &outroot.&modeltype._cf92_4";
x "mkdir &outroot.&modeltype._cf92_5";
x "mkdir &outroot.&modeltype._cf92_6";
x "mkdir &outroot.&modeltype._cf92_7";
x "mkdir &outroot.&modeltype._cf92_8";
x "mkdir &outroot.&modeltype._cf92_9";
x "mkdir &outroot.&modeltype._cf92_10";
x "mkdir &outroot.&modeltype._cf92_11";
x "mkdir &outroot.&modeltype._cf92_12";
x "mkdir &outroot.&modeltype._cfl";
option xwait;

/* 1: no time limit */
%let outpath=&outroot.&modeltype._cf92_1\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute_92 tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/* 2: no work requirement */
%let outpath=&outroot.&modeltype._cf92_2\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1_92 sanction2_92 sanction3_92)(13.7);	
run;

/* 3: welfare benefit disregards */
%let outpath=&outroot.&modeltype._cf92_3\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol_92 benpct_92 bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/* 4: EITC */
%let outpath=&outroot.&modeltype._cf92_4\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1_92 eitcb2_92 eitcb3_92 eitcbm_92 eitcr1_92 eitcr2_92
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc_92 seitcref_92 seitcp_92
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/* 5: minimum wage */
%let outpath=&outroot.&modeltype._cf92_5\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj_92)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/* 6: local unemployment rate */
%let outpath=&outroot.&modeltype._cf92_6\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value_92 valuel3_92 valuel6_92 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/* 7: local industry and wage structure */
%let outpath=&outroot.&modeltype._cf92_7\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100_92 empshare_8_92)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/* 8: demographic variables constant */
%let outpath=&outroot.&modeltype._cf92_8\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18_92)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white_92 tage_92 hs_92 south_92 rfoklt18_92 mtagecurrent_92 y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white_92 tage_92 hs_92 south_92 rfoklt18_92 mtagecurrent_92 y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white_92 tage_92 hs_92 south_92 rfoklt18_92 mtagecurrent_92 y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white_92 tage_92 hs_92 south_92 y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white_92 tage_92 hs_92 south_92 y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/* 9: Other changes */
%let outpath=&outroot.&modeltype._cf92_9\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold_92 bestd_92 bemax_92 bendol benpct bc_bepct_92
			fsge_92 fsne_92 fsmb_92 hded_92
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1_92 ded_92
			stax_92 nbracket_92 staxdedttl_92 staxb1_92 staxb2_92 staxb3_92 staxb4_92 staxb5_92 staxb6_92 staxr1_92 staxr2_92 staxr3_92
			staxr4_92 staxr5_92 staxr6_92 staxmb1_92 staxmb2_92 staxmb3_92 staxmb4_92 staxmb5_92 staxmb6_92 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost_92 diver_92)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert_92 fsebt_92 )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro_92 avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro_92 value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/* 10: all of the above */
%let outpath=&outroot.&modeltype._cf92_10\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute_92 tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold_92 bestd_92 bemax_92 bendol_92 benpct_92 bc_bepct_92
			fsge_92 fsne_92 fsmb_92 hded_92
			eitcb1_92 eitcb2_92 eitcb3_92 eitcbm_92 eitcr1_92 eitcr2_92
			ftaxr1_92 ded_92
			stax_92 nbracket_92 staxdedttl_92 staxb1_92 staxb2_92 staxb3_92 staxb4_92 staxb5_92 staxb6_92 staxr1_92 staxr2_92 staxr3_92
			staxr4_92 staxr5_92 staxr6_92 staxmb1_92 staxmb2_92 staxmb3_92 staxmb4_92 staxmb5_92 staxmb6_92 seitc_92 seitcref_92 seitcp_92
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj_92)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18_92)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white_92 tage_92 hs_92 south_92 rfoklt18_92 mtagecurrent_92 y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white_92 tage_92 hs_92 south_92 rfoklt18_92 mtagecurrent_92 y9495 y9697 y9899 tanfcost_92 diver_92)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white_92 tage_92 hs_92 south_92 rfoklt18_92 mtagecurrent_92 y9495 y9697 y9899 fsrecert_92 fsebt_92 )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white_92 tage_92 hs_92 south_92 y9495 y9697 y9899 metro_92 avgwage_0100_92 empshare_8_92)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white_92 tage_92 hs_92 south_92 y9495 y9697 y9899 metro_92 value_92 valuel3_92 valuel6_92 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1_92 sanction2_92 sanction3_92)(13.7);	
run;

/* 11: all policies (exclude "other changes") */
%let outpath=&outroot.&modeltype._cf92_11\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute_92 tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol_92 benpct_92 bc_bepct
			fsge fsne fsmb hded
			eitcb1_92 eitcb2_92 eitcb3_92 eitcbm_92 eitcr1_92 eitcr2_92
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc_92 seitcref_92 seitcp_92
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj_92)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1_92 sanction2_92 sanction3_92)(13.7);	
run;

/* 12: all econ (exclude "other changes") */
%let outpath=&outroot.&modeltype._cf92_12\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100_92 empshare_8_92)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value_92 valuel3_92 valuel6_92 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;

/**************************************************************************************************/
/* 	long-run simulation (output only 1 observation per individual) */
/* keep everything in 1992 / at wave 1 level */
%let outpath=&outroot.&modeltype._cfl\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put "  1" (horizon)(3.0) "  0" "  1" " 15" (horizon)(3.0) "  0" "  0" " 15";
	* gcount horizon tl_compute tlstart tllen tlhor tlred tlnlife tlnlbase;
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold_92 bestd_92 bemax_92 bendol_92 benpct_92 bc_bepct_92
			fsge_92 fsne_92 fsmb_92 hded_92
			eitcb1_92 eitcb2_92 eitcb3_92 eitcbm_92 eitcr1_92 eitcr2_92
			ftaxr1_92 ded_92
			stax_92 nbracket_92 staxdedttl_92 staxb1_92 staxb2_92 staxb3_92 staxb4_92 staxb5_92 staxb6_92 staxr1_92 staxr2_92 staxr3_92
			staxr4_92 staxr5_92 staxr6_92 staxmb1_92 staxmb2_92 staxmb3_92 staxmb4_92 staxmb5_92 staxmb6_92 seitc_92 seitcref_92 seitcp_92
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj_92)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent notimedummy notimedummy notimedummy)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent notimedummy notimedummy notimedummy tanfcost_92 diver_92)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent notimedummy notimedummy notimedummy fsrecert_92 fsebt_92 )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south notimedummy notimedummy notimedummy metro_92 avgwage_0100_92 empshare_8_92)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south notimedummy notimedummy notimedummy metro_92 value_92 valuel3_92 valuel6_92 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1_92 sanction2_92 sanction3_92)(13.7);	
	where swave=2;
run;

/**************************************************************************************************/
/* Result not reported in tables */
/* assume everyone in 1996 panel is subject to an intermediate work requirement sanction from wave 1 */
option noxwait;
x "mkdir &outroot.&modeltype._cf92_13";
option xwait;
data panel;
	set panel;
	sanction1_cf=sanction1;
	sanction2_cf=sanction2;
	sanction3_cf=sanction3;
	if id<10000 then do;
		sanction1_cf=0;
		sanction2_cf=1;
		sanction3_cf=0;
	end;
run;

%let outpath=&outroot.&modeltype._cf92_13\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1 eitcb2 eitcb3 eitcbm eitcr1 eitcr2
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc seitcref seitcp
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1_cf sanction2_cf sanction3_cf)(13.7);	
run;

/**************************************************************************************************/
/* Result not reported in tables */
/* assume no tl, no wr, or no eitc (two or three removed at a time) */
option noxwait;
x "mkdir &outroot.&modeltype._cf92_14";
option xwait;
%let outpath=&outroot.&modeltype._cf92_14\;		 			* output file directory;
data _null_;	* group data;
	set panelg;
	file "&outpath.gdata0id.txt" lrecl=2000;
	put (id) (8.0);
	file "&outpath.gdata0.txt" lrecl=2000;
	put (gcount horizon tlcompute_92 tlstart tllen tlhor tlred tlnlife tlnlbase) (3.0);
run;
data _null_;	* panel data;
	set panel;
	file "&outpath.gdata1.txt" lrecl=2000;
	put (choice) (2.0);
	file "&outpath.gdata2.txt" lrecl=2000;
	put (wage) (13.7);
	file "&outpath.gdata3.txt" lrecl=2000; 	* program parameters (including state tax parameters);
	put (threshold bestd bemax bendol benpct bc_bepct
			fsge fsne fsmb hded
			eitcb1_92 eitcb2_92 eitcb3_92 eitcbm_92 eitcr1_92 eitcr2_92
			ftaxr1 ded
			stax nbracket staxdedttl staxb1 staxb2 staxb3 staxb4 staxb5 staxb6 staxr1 staxr2 staxr3
			staxr4 staxr5 staxr6 staxmb1 staxmb2 staxmb3 staxmb4 staxmb5 staxmb6 seitc_92 seitcref_92 seitcp_92
		) (13.7);
	file "&outpath.gdata4.txt" lrecl=2000;	* dummy dataset;
	put (yearnew)(13.7);
	file "&outpath.gdata5.txt" lrecl=2000;
	put (fedminwageadj)(13.7);
	file "&outpath.gdata6.txt" lrecl=2000;
	put (rfoklt18)(13.7);
	file "&outpath.gdata7.txt" lrecl=2000; 	* cumulative variables (including cumhitwait);
	put (cumafdc hit cumhitwait) (3.0);
	file "&outpath.gdata4work.txt" lrecl=2000;
	put (LAGWORK white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899)(13.7); *10 var;
	file "&outpath.gdata4afdc.txt" lrecl=2000;
	put (LAGAFDC white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 tanfcost diver)(13.7); *12 var;
	file "&outpath.gdata4fs.txt" lrecl=2000;
	put (LAGFS white tage hs south rfoklt18 mtagecurrent y9495 y9697 y9899 fsrecert fsebt )(13.7); *12 var;
	file "&outpath.gdata4wage.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro avgwage_0100 empshare_8)(13.7); *11 var;
	file "&outpath.gdata4job.txt" lrecl=2000;
	put (LAGWORK white tage hs south y9495 y9697 y9899 metro value valuel3 valuel6 )(13.7);	*12 var;
	file "&outpath.gdata4sanction.txt" lrecl=2000; 	* sanction variables (recoded);
	put (sanction1 sanction2 sanction3)(13.7);	
run;
