/* Marc Chan 2012/04/02 */
/* Creates selected tables for time limit paper */
options pagesize=40 font='courier new' 8;
libname dataout "i:\tl2010";
%let outroot=i:\tl2010\fortrancode\basemodel\;

data fstatic_desc;
	set dataout.fstatic;
	array staxb{*} staxb1-staxb6;
	array staxr{*} staxr1-staxr6;
	array staxmb{*} staxmb1-staxmb6;
	array staxbd{*} staxbd1-staxbd6;
	array staxrd{*} staxrd1-staxrd6;
	array staxmbd{*} staxmbd1-staxmbd6;
	if hour=0 then hour_desc=.;			* workers only;
	else hour_desc=hour;
	if wage=0 then wage_desc=.;
	else wage_desc=wage/80;
	tllife=(tlnlife=0 & tlred=0 & tl=1);
	if tlcurrent=0 then do;
		tlredcurrent=0;
		tlnlifecurrent=0;
		tllifecurrent=0;
		tllen_desc=.;
	end;
	else do;
		tlredcurrent=tlred;
		tlnlifecurrent=tlnlife;
		tllifecurrent=tllife;
		tllen_desc=tllen/3;		* exposed to time limit only;
	end;
	if tlnlifecurrent=0 then tlnlbase_desc=.;
	else tlnlbase_desc=tlnlbase/3; * exposed to non-life limit only;
	if seitc=0 then seitcp_desc=.;
	else seitcp_desc=seitcp;
	nbrackettmp=max(nbracket,1);
	do iloop=1 to nbrackettmp;
		staxbd(iloop)=staxb(iloop);
		staxrd(iloop)=staxr(iloop);
		staxmbd(iloop)=staxmb(iloop);
		if staxbd(iloop)=8333 then do; * set last bracket to missing;
			staxbd(iloop)=.;
			staxmbd(iloop)=.;
		end;
	end;
	do iloop=1 to nbrackettmp;
		if staxrd(iloop)=staxr(iloop); * set rate ;
	end;
	south=south*100;
	hs=hs*100;
	white=white*100;
	metro=metro*100;
	empshare_8=empshare_8*100;
	diver=diver*100;
	fsrecert=fsrecert*100;
	fsebt=fsebt*100;
	benpct=benpct*100;
	eitcr1=eitcr1*100;
	eitcr2=eitcr2*100;
	stax=stax*100;
	staxrd1=staxrd1*100;
	staxrd2=staxrd2*100;
	staxrd3=staxrd3*100;
	staxrd4=staxrd4*100;
	seitc=seitc*100;
	seitcref=seitcref*100;
	seitcp_desc=seitcp_desc*100;
	wr =  (sanction2=1 | sanction3=1); * table 5;
	sanction1=sanction1*100;
	sanction2=sanction2*100;
	sanction3=sanction3*100;
	tllifecurrent=tllifecurrent*100;
	tlnlifecurrent=tlnlifecurrent*100;
	tlredcurrent=tlredcurrent*100;
	hit=hit*100;
	workfull=(choice in (3,8))*100;
	work=work*100;
	afdc=afdc*100;
	fs=fs*100;
	panel96=(id<10000);
	fsnew=(choice in (4,5,6,7,8))*100;
	afdcnew=(choice in (4,5))*100;
	worknew=(choice in (2,3,5,7,8))*100;
	format _numeric_ 8.2;
run;

data fstatic_desc;
	set fstatic_desc;
	label
	south="Census south region residence"
	hs="Attend grade 12 or above"
	tage="Mother's age"
	white="White"
	rfoklt18="Number of children less than 18"
	mtagecurrent="Age of youngest child"
	metro="Metro area residence"
	hour="Weekly raw work hours";
run;

/* Table 2: summary statistics of selected variables */
ods html;
proc means data=fstatic_desc nolabel maxdec=1;
format _numeric_ 8.2;
class panel96;
var south hs tage white rfoklt18 mtagecurrent metro 
hour_desc wage_desc
value avgwage_0 empshare_8 
tanfcost diver fsrecert fsebt
bemax bendol benpct threshold fsmb hded fsne 
eitcbm eitcr1 eitcr2 eitcb1 eitcb2 eitcb3 
ded 
stax staxdedttl	nbracket
staxbd1--staxbd3 staxrd1--staxrd3 
seitc seitcref seitcp_desc
exemption sanction1 sanction2 sanction3
tllifecurrent tlnlifecurrent tlredcurrent tllen_desc tlnlbase_desc hit;
*where id<10000;
run;
ods html close;

/*********************** not reported in table - descriptive statistics ******************/
data temp;
	set dataout.fgstatic;
	tl= (tllen ne 0 | tlnlife=1 | tlred=1);
	tlmon_desc=tlmon;
	if tl=0 then tlmon_desc=.;	
	tllen_desc=tllen/3;
	if tl=0 then tllen_desc=.;
	tlnlbase_desc=tlnlbase/3;
	if tlnlife=0 then tlnlbase_desc=.;
	panel96=(id<10000);
run;

data temp;
	set fstatic_desc;
	if tlcurrent=1;
run;
data temp;
	set temp;
	by id swave;
	if first.id;
	keep id swave tlmon panel96 currentmon;
run;
proc means data=temp;
	class panel96;
	var tlmon currentmon;
run;

data temp;
	set fstatic_desc;
	if sanctionid>0;
run;
data temp;
	set temp;
	by id swave;
	if first.id;
	keep id swave currentmon panel96;
run;
proc means data=temp;
	class panel96;
	var currentmon;
run;

proc means data=temp nolabel;
	format _numeric_ 8.2;
	class panel96;
	var tl tlmon_desc tllen_desc tlnlbase_desc;
run;

data fgstatic_desc;
	set dataout.fgstatic;
run;

data temp999;
	set fstatic_desc;
	keep id swave nbracket stax staxb1--staxb6 staxr1--staxr6 staxmb1--staxmb6;
	*keep tlcurrent tl tlred tlnlife tlnlbase tllen id swave tlredcurrent tlnlifecurrent tlnlbase_desc tllen_desc ;
run;

/* Table 3: Sample participation rates and choice distributions by year */
ods html style=minimal;
proc tabulate data=fstatic_desc;
	class yearnew choice;
	table (yearnew='' all), choice=''*pctn<choice>=''*f=6.2  N*f=5.0;
	format yearnew choice 5.0;  
	where swave>=2;
run;
ods html close;

/* Table 6: Sample participation rates by time limit and work requirement policies */
ods html style=minimal;
proc tabulate data=fstatic_desc;
	class tlcurrent wr;
	var worknew workfull afdcnew fsnew;
	table (worknew workfull afdcnew fsnew)*mean*f=6.2, tlcurrent=''*wr='' all;
	where swave>=2;
run;
ods html close;
proc tabulate data=fstatic_desc;
	class tlcurrent wr;
	table (tlcurrent*wr all)*pctn<tlcurrent*wr all>=''*f=6.1;
	where swave>=2;
run;

/* Figure 1: benefit functions */
proc means data=fstatic_desc;
	class year;
	var bemax bestd threshold bendol benpct fsmb fsne fsge hded
	eitcbm eitcr1 eitcr2 eitcb1 eitcb2 ded
	stax nbracket staxdedttl 
	staxrd1 staxrd2 staxrd3 staxrd4 staxbd1 staxbd2 staxbd3 staxbd4
	seitcp
;
	where year in (1992,1999);
run;

/* Figure 4: Data for External validation */
ods html;
proc tabulate data=fstatic_desc;
	class swave panel96;
	var worknew workfull afdcnew fsnew;
	table panel96=''*swave='', (worknew workfull='ft' afdcnew fsnew)*mean=''*f=6.1 / box='Panel96/Wave';
	format swave 5.0;
	where swave>=2;
run;
ods html close;

/* Figure A1: Distribution of raw weekly work hours for workers */
proc format;
	value hour_cat
	1-10 = "1 to 10"
	11-20 = "11 to 20"
	21-29 = "21 to 29"
	30-34 = "30 to 34"
	35-39 = "35 to 39"
	40 = "40"
	41-50 = "41-50"
	51-200 = "51+";
run;
ods html style=minimal;
proc tabulate data=fstatic_desc;
	class hour panel96;
	table hour='', panel96=''*pctN<hour>='';
	format hour hour_cat.;
	where hour>0;
run;
ods html close;
